from MAST.structopt.tools.lammps import LAMMPS
from ase.calculators.vasp import Vasp
from ase.calculators.lj import LennardJones
import os
try:
    from mpi4py import MPI
except ImportError:
    pass
import logging

def setup_calculator(Optimizer):
    """
    Subprogram to set up the ASE style calculator for a system
    Input:
        Optimizer  = Optimizer class object with parameters for setting up calculator
    Output:
        calc = ASE Calculator object for running a calculation
    """
    if 'SetCalc' in Optimizer.debug:
        debug=True
        #logger = initialize_logger(Optimizer.loggername)
        logger = logging.getLogger(Optimizer.loggername)
    else:
        debug=False
    atomlist=Optimizer.atomlist
    atomlist=sorted(atomlist,key=lambda symbol: symbol[0])
    if Optimizer.calc_method=='VASP':
        if debug:
            logger.info('Setting up vasp calculator = {0}'.format(Optimizer.vaspcalc))
        calc = eval(Optimizer.vaspcalc)
    elif 'MAST' in Optimizer.calc_method:
        if debug:
            logger.info('Setting up dummy calculator for MAST')
        calc = LennardJones()
    elif Optimizer.calc_method=='LennardJones':
        if debug:
            logger.info('Setting up Lennard Jones calculator from ASE')
        calc = LennardJones()
    else:
        if Optimizer.pair_style=='tersoff':
            if debug:
                logger.info('Setting up LAMMPS calculator with Tersoff potential')
            parcoff = '* * {0}'.format(Optimizer.pot_file)
            for one in atomlist:
                parcoff+=' {0}'.format(one[0])
            pair_coeff = [parcoff]
            mass = ['1 {0}'.format(atomlist[0][2])]
            if len(atomlist) > 1:
                for i in range(len(atomlist)-1):
                    mass.append('{0} {1}'.format(i+2,atomlist[i+1][2]))
            parameters = { 'pair_style' : Optimizer.pair_style, \
            'pair_coeff' : pair_coeff , 'mass' : mass }
            filesL = [ Optimizer.pot_file ]
        elif Optimizer.pair_style=='eam':
            if debug:
                logger.info('Setting up LAMMPS calculator with EAM potential')
            pair_coeff = [ '* * {0}'.format(Optimizer.pot_file)]
            parameters = { 'pair_style' : Optimizer.pair_style, 'pair_coeff' : pair_coeff }
            filesL = [ Optimizer.pot_file ]
        elif Optimizer.pair_style=='eam/fs':
            if debug:
                logger.info('Setting up LAMMPS calculator with EAM/FS potential')
            parcoff = '* * {0}'.format(Optimizer.pot_file)
            for one in atomlist:
                parcoff+=' {0}'.format(one[0])
            pair_coeff = [parcoff]
            mass = ['1 {0}'.format(atomlist[0][2])]
            if len(atomlist) > 1:
                for i in range(len(atomlist)-1):
                    mass.append('{0} {1}'.format(i+2,atomlist[i+1][2]))
            parameters = { 'pair_style' : Optimizer.pair_style, 
            'pair_coeff' : pair_coeff , 'mass' : mass }
            filesL = [ Optimizer.pot_file ]
        elif Optimizer.pair_style=='eam/cd':
            if debug:
                logger.info('Setting up LAMMPS calculator with EAM/CD potential')
            parcoff = '* * {0}'.format(Optimizer.pot_file)
            for one in atomlist:
                parcoff+=' {0}'.format(one[0])
            pair_coeff = [parcoff]
            parameters = { 'pair_style' : Optimizer.pair_style, 'pair_coeff' : pair_coeff}
            filesL = [ Optimizer.pot_file ]
        elif Optimizer.pair_style=='edip':
            if debug:
                logger.info('Setting up LAMMPS calculator with EDIP potential')
            parcoff = '* * {0}'.format(Optimizer.pot_file)
            for one in atomlist:
                parcoff+=' {0}'.format(one[0])
            pair_coeff = [parcoff]
            mass = ['1 {0}'.format(atomlist[0][2])]
            if len(atomlist) > 1:
                for i in range(len(atomlist)-1):
                    mass.append('{0} {1}'.format(i+2,atomlist[i+1][2]))
            parameters = { 'pair_style' : Optimizer.pair_style, \
            'pair_coeff' : pair_coeff , 'mass' : mass, 'newton': 'on' }
            filesL = [ Optimizer.pot_file ]
        elif Optimizer.pair_style=='bop':
            if debug:
                logger.info('Setting up LAMMPS calculator with BOP potential')
            parcoff = '* * {0}'.format(Optimizer.pot_file)
            for one in atomlist:
                parcoff+=' {0}'.format(one[0])
            parcoff+='\ncommunicate single cutoff {0}'.format(Optimizer.bopcutoff)
            pair_coeff = [parcoff]
            mass = ['1 {0}'.format(atomlist[0][2])]
            if len(atomlist) > 1:
                for i in range(len(atomlist)-1):
                    mass.append('{0} {1}'.format(i+2,atomlist[i+1][2]))
            parameters = { 'pair_style' : Optimizer.pair_style, \
            'pair_coeff' : pair_coeff, 'mass' : mass, 'newton': 'on' }
            filesL = [ Optimizer.pot_file ]
        elif Optimizer.pair_style=='buck':
            if debug:
                logger.info('Setting up LAMMPS calculator with Buckingham potential')
            pairstyle='{0} {1}'.format(Optimizer.pair_style, Optimizer.buckcutoff)
            pair_coeff=Optimizer.buckparameters
            mass = ['1 {0}'.format(atomlist[0][2])]
            if len(atomlist) > 1:
                for i in range(len(atomlist)-1):
                    mass.append('{0} {1}'.format(i+2, atomlist[i+1][2]))	
            parameters = {'pair_style': pairstyle, 'pair_coeff': pair_coeff, \
             'mass' : mass }
            filesL=None
        elif Optimizer.pair_style=='other':
            """WARNING: This style still needs work. Intended to allow user flexibility with potential specification"""
            if debug:
                logger.info('Setting up LAMMPS calculator with user input potential')
            mass = ['1 {0}'.format(atomlist[0][2])]
            if len(atomlist) > 1:
                for i in range(len(atomlist)-1):
                    mass.append('{0} {1}'.format(i+2,atomlist[i+1][2]))
            if Optimizer.ps_other!=None:
                if 'newton' in Optimizer.ps_other:
                    parameters = {'pair_style' : Optimizer.ps_name, \
                    'pair_coeff': [Optimizer.pair_coeff], 'mass': mass,'newton':'on'}
                else:
                    parameters = {'pair_style' : Optimizer.ps_name, \
                    'pair_coeff': [Optimizer.pair_coeff], 'mass': mass,'newton':'off'}
                if 'charges' in Optimizer.ps_other:
                    cs=Optimizer.ps_other.split('charges:')
                    parameters['mass'][len(parameters['mass'])-1] +=cs[1]
                    parameters['newton']+='\natom_style charge'
            else:
                parameters = {'pair_style' : Optimizer.ps_name, \
                'pair_coeff': Optimizer.pair_coeff, 'mass': mass}
            if Optimizer.pot_file !=None:
                filesL=Optimizer.pot_file
            else:
                filesL=None
        else:
            if debug:
                logger.warn('No LAMMPS potential recognized. Setting up LAMMPS calculator with Lennard Jones potential')
            parameters={}
            filesL=None
            print 'WARNING: No LAMMPS potential recognized. Assuming Lennard Jones Potential'
        if Optimizer.lammps_min != None:
            if debug:
                logger.info('Adding local energy minimizer to LAMMPS calculator')
            try:
                parameters['mass'][len(parameters['mass'])-1] += '\nmin_style {0}'.format(Optimizer.lammps_min_style)
            except KeyError:
                parameters['pair_coeff'][0] += '\nmin_style {0}'.format(Optimizer.lammps_min_style)
            parameters['minimize'] = Optimizer.lammps_min
        parameters['thermosteps'] = Optimizer.lammps_thermo_steps
        if Optimizer.lammps_keep_files:
            if debug:
                logger.info('Setting up directory for keeping LAMMPS files')
            try:
                rank = MPI.COMM_WORLD.Get_rank()
                if Optimizer.parallel:
                    if 'Island_Method' not in Optimizer.algorithm_type:
                        real_rank = MPI.COMM_WORLD.Get_rank()
                        rank = 0
                        path = os.path.join(os.getcwd(),'{0}-rank{1}'.format(Optimizer.filename,rank))
                        if not os.path.exists(os.path.join(path,'LAMMPSFiles')):
                            os.mkdir(os.path.join(path,'LAMMPSFiles'))
                            logger.info('Making directory: {0}'.format(os.path.join(path,'LAMMPSFiles')))
            except:
                rank = 0
            if filesL != None:
                path = os.path.join(os.getcwd(),'{0}-rank{1}'.format(Optimizer.filename,rank))
                if Optimizer.parallel and ('Island_Method' not in Optimizer.algorithm_type):
                    tmpdir = os.path.join(os.path.join(path, 'LAMMPSFiles'),'rank-{0}'.format(real_rank))
                    calc = LAMMPS(parameters=parameters, files=filesL, \
                        keep_tmp_files=True, tmp_dir=tmpdir)
                else:
                    calc = LAMMPS(parameters=parameters, files=filesL, \
                        keep_tmp_files=True, tmp_dir=os.path.join(path, 'LAMMPSFiles'))
            else:
                path = os.path.join(os.getcwd(),'{0}-rank{1}'.format(Optimizer.filename,rank))
                calc = LAMMPS(parameters=parameters, keep_tmp_files=True, \
                    tmp_dir=os.path.join(path,'LAMMPSFiles'))
        else:
            if filesL != None: 
                calc = LAMMPS(parameters=parameters, files=filesL)
            else:
                calc = LAMMPS(parameters=parameters)
    return calc



import numpy

def shift_atoms(atoms, position=None):
    """Function to shift the positions of the atoms so that
    the given location is in the center of the cell.
    Input:
        atoms = ASE atoms object to be shifted
        position = [x,y,z] position to shift to center of cell
            default is center of mass of atoms
    Output:
        atoms = ASE atoms object that has been shifted
    """
    atoms = atoms.copy()
    if not position:
        position = atoms.get_center_of_mass()
    cell = numpy.maximum.reduce(atoms.get_cell())
    trans = [cell[i]/2.0-position[i] for i in range(3)]
    atoms.translate(trans)
    positions = atoms.get_positions()
    for i in range(len(positions)):
        if atoms.pbc[0]:
            if trans[0] > 0:
                if positions[i][0] > cell[0]:
                    positions[i][0] -= cell[0]
            else:
                if positions[i][0] < 0:
                    positions[i][0] += cell[0]
        if atoms.pbc[1]:
            if trans[1] > 0:
                if positions[i][1] > cell[1]:
                    positions[i][1] -= cell[1]
            else:
                if positions[i][1] < 0:
                    positions[i][1] += cell[1]
        if atoms.pbc[2]:
            if trans[2] >0:
                if positions[i][2] > cell[2]:
                    positions[i][2] -= cell[2]
            else:
                if positions[i][2] < 0:
                    positions[i][2] += cell[2]
    atoms.set_positions(positions)
    return atoms




from MAST.structopt.tools import get_best
from MAST.structopt.switches import moves_switch, lambdacommamu
from MAST.structopt.generate import gen_pop_box
from MAST.structopt.generate.Individual import Individual
from MAST.structopt.tools import remove_duplicates

def fitpred_bests(pop,Optimizer):
    """Predator function to identify similar structures based on energy and replace one 
    with structure from BESTS List.
    """
    fitlist = [one.fitness for one in pop]
    nfitlist, nindices = remove_duplicates(fitlist, Optimizer.demin)
    STR = ''
    newpop = []
    if len(nfitlist) != len(fitlist):
        STR+='Predator: Removed total of '+repr(len(fitlist)-len(nfitlist))+' from population\n'
    otherlist = []
    for i in range(len(pop)):
        if i not in nindices:
            STR+='Predator: Removed '+repr(pop[i].history_index)+'\n'
            otherlist.append(pop[i])
        else:
            newpop.append(pop[i])
    count = 0
    while len(newpop) < Optimizer.nindiv:
        try:
            Optimizer.BESTS
        except:
            Optimizer.BESTS=[]
        if len(Optimizer.BESTS) > 0:
            idx = random.choice(range(len(Optimizer.BESTS)))
            newpop.append(Optimizer.BESTS[idx])
            STR+='Predator: Adding in structure from Best List from position = {0} with fitness = {1}\n'.format(idx,Optimizer.BESTS[idx].fitness)
            newindices.append(len(pop)+count)
            count+=1
        else:
            indiv = random.choice(otherlist).duplicate()
            indiv, scheme = moves_switch(indiv,Optimizer)
            indiv.energy = 1000
            indiv.fitness = 1000
            newpop.append(indiv)
            STR+='Predator: Adding mutated duplicates to new pop history='+indiv.history_index+'\n'
            nindices.append(indiv.index)
    nindices.sort()
    if Optimizer.natural_selection_scheme=='fussf':
        for ind in newpop:
            if ind.fingerprint == 0:
                ind.fingerprint = get_fingerprint(Optimizer,ind,Optimizer.fpbin,Optimizer.fpcutoff)
    if 'lambda,mu' in Optimizer.algorithm_type:
        try:
            mark = [ index for index,n in enumerate(nindices) if n > Optimizer.nindiv-1][0]
        except:
            mark = Optimizer.nindiv
        Optimizer.mark = mark
        pop, str1 = lambdacommamu.lambdacommamu(newpop, Optimizer)
        STR+=str1
    else:
        pop = selection_switch(newpop, Optimizer.nindiv, Optimizer.natural_selection_scheme, Optimizer)
    pop = get_best(pop,len(pop))
    return pop, STR



"""Tests for Vaspnebchecker"""

from MAST.ingredients.checker.vaspnebchecker import VaspNEBChecker
import unittest
from unittest import SkipTest
import os
import time
import MAST
import pymatgen
from MAST.utility import dirutil
from MAST.utility import MASTFile
import shutil
testname="checker_test_vaspneb"
testdir = dirutil.get_test_dir(testname)

class TestVaspnebchecker(unittest.TestCase):

    def setUp(self):
        os.chdir(testdir)
        if not os.path.isdir('childdir'):
            os.mkdir('childdir')
        shutil.copy("files/metadata.txt","childdir")

    def tearDown(self):
        #return
        dnames = dirutil.walkdirs("childdir",1,1)
        for dname in dnames:
            shutil.rmtree(dname)
        fnames = dirutil.walkfiles("childdir",1,1)
        for fname in fnames:
            os.remove(fname)


    def test_get_path_to_write_neb_parent_energy(self):
        kdict=dict()
        kdict['mast_neb_settings']=dict()
        kdict['mast_neb_settings']['images']=3
        myvcneb=VaspNEBChecker(name="childdir",program_keys = kdict)
        mypath = myvcneb.get_path_to_write_neb_parent_energy(1)
        self.assertEqual(mypath, "childdir/00/OSZICAR")
        mypath = myvcneb.get_path_to_write_neb_parent_energy(2)
        self.assertEqual(mypath, "childdir/04/OSZICAR")
        mypath = myvcneb.get_path_to_write_neb_parent_energy('02')
        self.assertEqual(mypath, "childdir/02/OSZICAR")
        #self.testclass.get_path_to_write_neb_parent_energy(myimages, parent)

    def test_set_up_neb_folders_no_mast_coordinates(self):
        mystrs=list()
        pos=dict()
        for posstr in ['00','01','02','03','04']:
            pos[posstr] = pymatgen.io.vaspio.Poscar.from_file("structures/POSCAR_%s" % posstr)
            mystrs.append(pos[posstr].structure)
        kdict=dict()
        kdict['images']=3
        myvcneb=VaspNEBChecker(name="childdir",program_keys=kdict)
        myvcneb.set_up_neb_folders(mystrs)
        for subdir in ['00','01','02','03','04']:
            mypos = pymatgen.io.vaspio.Poscar.from_file("childdir/POSCAR_%s" % subdir)
            self.assertEqual(mypos.structure,pos[subdir].structure)
            self.assertEqual(mypos.structure.lattice,pos[subdir].structure.lattice)
            self.assertEqual(mypos.structure.sites,pos[subdir].structure.sites)
            mypos = pymatgen.io.vaspio.Poscar.from_file("childdir/%s/POSCAR" % subdir)
            self.assertEqual(mypos.structure,pos[subdir].structure)
            self.assertEqual(mypos.structure.lattice,pos[subdir].structure.lattice)
            self.assertEqual(mypos.structure.sites,pos[subdir].structure.sites)
        #self.testclass.set_up_neb_folders(image_structures)
    def test_set_up_neb_folders_with_mast_coordinates(self):
        kdict=dict()
        kdict['mast_neb_settings']=dict()
        kdict['mast_neb_settings']['images']=3
        kdict['mast_coordinates']=["structures/POSCAR_coords_01","structures/POSCAR_coords_02","structures/POSCAR_coords_03"]
        mystrs=list()
        pos=dict()
        graftedpos=dict()
        for posstr in ['00','01','02','03','04']:
            graftedpos[posstr] = pymatgen.io.vaspio.Poscar.from_file("structures/POSCAR_grafted_%s" % posstr)
            pos[posstr] = pymatgen.io.vaspio.Poscar.from_file("structures/POSCAR_%s" % posstr)
            mystrs.append(pos[posstr].structure)
        myvcneb=VaspNEBChecker(name="childdir",program_keys=kdict)
        myvcneb.set_up_neb_folders(mystrs)
        for subdir in ['00','01','02','03','04']:
            mypos = pymatgen.io.vaspio.Poscar.from_file("childdir/POSCAR_%s" % subdir)
            self.assertEqual(mypos.structure,graftedpos[subdir].structure)
            self.assertEqual(mypos.structure.lattice,graftedpos[subdir].structure.lattice)
            self.assertEqual(mypos.structure.sites,graftedpos[subdir].structure.sites)
            mypos = pymatgen.io.vaspio.Poscar.from_file("childdir/%s/POSCAR" % subdir)
            self.assertEqual(mypos.structure,graftedpos[subdir].structure)
            self.assertEqual(mypos.structure.lattice,graftedpos[subdir].structure.lattice)
            self.assertEqual(mypos.structure.sites,graftedpos[subdir].structure.sites)
        #self.testclass.set_up_neb_folders(image_structures)

    def test_is_complete(self):
        kdict=dict()
        kdict['mast_neb_settings']=dict()
        kdict['mast_neb_settings']['images']=3
        myvcneb=VaspNEBChecker(name="notready1",program_keys=kdict)
        self.assertFalse(myvcneb.is_complete())
        myvcneb=VaspNEBChecker(name="ready",program_keys=kdict)
        self.assertFalse(myvcneb.is_complete())
        myvcneb=VaspNEBChecker(name="started",program_keys=kdict)
        self.assertFalse(myvcneb.is_complete())
        myvcneb=VaspNEBChecker(name="done",program_keys=kdict)
        self.assertTrue(myvcneb.is_complete())
        #self.testclass.is_complete()

    def test_is_ready_to_run(self):
        kdict=dict()
        kdict['mast_neb_settings']=dict()
        kdict['mast_neb_settings']['images']=3
        myvcneb=VaspNEBChecker(name="notready1",program_keys=kdict)
        self.assertFalse(myvcneb.is_ready_to_run())
        myvcneb=VaspNEBChecker(name="notready2",program_keys=kdict)
        self.assertFalse(myvcneb.is_ready_to_run())
        myvcneb=VaspNEBChecker(name="notready3",program_keys=kdict)
        self.assertFalse(myvcneb.is_ready_to_run())
        myvcneb=VaspNEBChecker(name="notready4",program_keys=kdict)
        self.assertFalse(myvcneb.is_ready_to_run())
        myvcneb=VaspNEBChecker(name="notready5",program_keys=kdict)
        self.assertFalse(myvcneb.is_ready_to_run())
        myvcneb=VaspNEBChecker(name="notready6",program_keys=kdict)
        self.assertFalse(myvcneb.is_ready_to_run())
        myvcneb=VaspNEBChecker(name="ready", program_keys=kdict)
        self.assertTrue(myvcneb.is_ready_to_run())
        myvcneb=VaspNEBChecker(name="started", program_keys=kdict)
        self.assertTrue(myvcneb.is_ready_to_run())
        myvcneb=VaspNEBChecker(name="done", program_keys=kdict)
        self.assertTrue(myvcneb.is_ready_to_run())
        
        #self.testclass.is_ready_to_run()

    def test__vasp_incar_setup(self):
        my_poscar = pymatgen.io.vaspio.Poscar.from_file("structures/POSCAR_00")
        my_structure = my_poscar.structure
        my_poscar.write_file("childdir/POSCAR")
        kdict=dict()
        kdict['mast_xc'] = "pw91"
        kdict['mast_pp_setup']={'Cr':'Cr_pv','Fe':'Fe_sv','Ni':'Ni_pv'}
        kdict['mast_neb_settings']=dict()
        kdict['mast_neb_settings']['images']=3
        kdict['IBRION'] = '1'
        kdict['POTIM'] = '0.5'
        kdict['LCLIMB'] = 'True'
        kdict['SPRING'] ='-5'
        myvc = VaspNEBChecker(name="childdir",program_keys=kdict,structure=my_structure)
        mypot = myvc._vasp_potcar_setup(my_poscar)
        myvc._vasp_incar_setup(mypot, my_poscar)
        myvc._vasp_neb_incar_modify()
        myincar = pymatgen.io.vaspio.Incar.from_file(os.path.join(testdir,"childdir","INCAR"))
        incar_compare = pymatgen.io.vaspio.Incar.from_file(os.path.join(testdir,"files","INCAR"))
        self.assertEqual(myincar, incar_compare)
        #self.testclass._vasp_incar_setup(my_potcar, my_poscar)

    def test_set_up_program_input(self):
        mystrs=list()
        pos=dict()
        for posstr in ['00','01','02','03','04']:
            pos[posstr] = pymatgen.io.vaspio.Poscar.from_file("structures/POSCAR_%s" % posstr)
            mystrs.append(pos[posstr].structure)
        kdict=dict()
        kdict['mast_kpoints']=[3,3,3,"G"]
        kdict['mast_xc']="pw91"
        kdict['mast_neb_settings']=dict()
        kdict['mast_neb_settings']['images']=3
        kdict['mast_neb_settings']['image_structures'] = mystrs
        myvcneb=VaspNEBChecker(name="childdir",program_keys=kdict)
        myvcneb.set_up_program_input()
        submitscript = MASTFile()
        submitscript.data="Submission script placeholder."
        submitscript.to_file("childdir/submit.sh")
        ozholder = MASTFile()
        ozholder.data="OSZICAR placeholder."
        submitscript.to_file("childdir/00/OSZICAR")
        submitscript.to_file("childdir/04/OSZICAR")
        self.assertTrue(myvcneb.is_ready_to_run())
        #self.testclass.set_up_program_input_neb(image_structures)

    def test_get_energy_from_energy_file(self):
        kdict=dict()
        kdict['mast_neb_settings']=dict()
        kdict['mast_neb_settings']['images']=3
        myvcneb=VaspNEBChecker(name="done",program_keys=kdict)
        estr = myvcneb.get_energy_from_energy_file()
        estr_compare="-99.860;-99.649;-99.538;-99.649;-99.860"
        self.assertEqual(estr,estr_compare)
        #self.testclass.get_energy_from_energy_file()

    def test_is_started(self):
        kdict=dict()
        kdict['images']=3
        myvcneb=VaspNEBChecker(name="notready1",program_keys=kdict)
        self.assertFalse(myvcneb.is_started())
        myvcneb=VaspNEBChecker(name="ready",program_keys=kdict)
        self.assertFalse(myvcneb.is_started())
        myvcneb=VaspNEBChecker(name="started",program_keys=kdict)
        self.assertTrue(myvcneb.is_started())
        myvcneb=VaspNEBChecker(name="done",program_keys=kdict)
        self.assertTrue(myvcneb.is_started())
        #self.testclass.is_started()




from MAST.structopt.inp_out import read_xyz
from MAST.structopt.generate.Individual import Individual
from MAST.structopt.tools import find_top_layer

def get_surface_restart_indiv(Optimizer, indiv):
    """
    Function to generate an structopt Individual class object containing 
        a surface structure from a previously existing structure
    Inputs:
        Optimizer = structopt Optimizer class
        indiv = ASE Atoms object containing the previously existing structure
    Outputs:
        individ = structopt Individual class object containing surface structure data
    """
    #Load surface structure
    surfs = read_xyz(Optimizer.surfacefile)
    cells = Optimizer.surfacecell
    surfs.set_cell(cells)
    surf.set_pbc([True,True,False])
    top,bulks=find_top_layer(indiv,Optimizer.surftopthick)
    individ=Individual(top)
    individ.bulki=bulks.copy()
    individ.bulko=bulks.copy()
    return individ



import random
import numpy
from ase import Atom, Atoms
from MAST.structopt.tools.find_defects import find_defects
from MAST.structopt.inp_out import write_xyz

def randalloybox(ind1, ind2, Optimizer):
    """Select a box in the alloy configuration
    *** Note: CX may be obsolete. In development ***
    """
    if 'CX' in Optimizer.debug:
        debug = True
    else:
        debug = False
    if Optimizer.structure != 'Defect':
        Optimizer.output.write('WARNING: Box Random alloy Cx attempted on nondefect structure. SKIPPING CX.\n')
    else:
        Optimizer.output.write('Box Random alloy Cx between individual '+repr(ind1.index)+' and individual '+repr(ind2.index)+'\n')
    
        #Perserve starting conditions of individual
        indi1 = ind1[0].copy()
        indi2 = ind2[0].copy()
        cell1 = numpy.maximum.reduce(indi1.get_cell())
        cell2 = numpy.maximum.reduce(indi2.get_cell())
        cell = numpy.minimum(cell1,cell2)
        pbc1 = indi1.get_pbc()
        pbc2 = indi2.get_pbc()

        #Build solids
        solid1 = indi1.copy()
        solid1.extend(ind1.bulki.copy())
        solid2 = indi2.copy()
        solid2.extend(ind2.bulki.copy())

        #Get starting concentrations and number of atoms
        nat1 = len(solid1)
        nat2 = len(solid2)
        symlist = list(set(solid1.get_chemical_symbols()))
        #Assumes same types of atoms in both solid1 and solid2
        concent1 = []
        concent2 = []
        for one in symlist:
            atmss = [atm for atm in solid1 if atm.symbol==one]
            concent1.append(len(atmss))
            atmss = [atm for atm in solid2 if atm.symbol==one]
            concent2.append(len(atmss))

        # Pick a origin point for box in the cell
        #pt=[random.uniform(0,cell[0]),random.uniform(0,cell[1]),random.uniform(0,cell[2])]
        pt1 = solid1[0].position
        pt2 = solid2[0].position

        #Find max radius of circle cut
        r1 = min([min(cell1[0]-pt1[0],pt1[0]),min(cell1[1]-pt1[1],pt1[1]),min(cell1[2]-pt1[2],pt1[2])])
        r2 = min([min(cell2[0]-pt2[0],pt2[0]),min(cell2[1]-pt2[1],pt2[1]),min(cell2[2]-pt2[2],pt2[2])])
        r = min(r1,r2)
        mcell = min(cell)
        if r > mcell*0.125:
            r = mcell*0.125
        if debug:
            Optimizer.output.write('Radius of box = '+repr(r)+'\nPosition in solid1 = '+repr(pt1)+'\nPosition in solid2 = '+repr(pt2)+'\n')

        #Find atoms within sphere of radius r
        solid1.append(Atom(position=pt1))
        dist1 = []
        blist1 = []
        for i in range(len(solid1)-1):
            d = solid1.get_distance(i,len(solid1)-1)
            if d < r:
                dist1.append((d,solid1[i]))
            else:
                blist1.append((d,solid1[i]))
        solid1.pop()

        solid2.append(Atom(position=pt2))
        dist2 = []
        blist2 = []
        for i in range(len(solid2)-1):
            d = solid2.get_distance(i,len(solid2)-1)
            if d < r:
                dist2.append((d,solid2[i]))
            else:
                blist2.append((d,solid2[i]))
        solid2.pop()

        #Translate spheres to opposite location
        dats1 = Atoms()
        for d,atm in dist1:
            dats1.append(atm)
        dats1.translate(-pt1)
        dats1.translate(pt2)
        dats2 = Atoms()
        for d,atm in dist2:
            dats2.append(atm)
        dats2.translate(-pt2)
        dats2.translate(pt1)

        #Exchange atoms in sphere and build new solids
        nsolid1 = dats2.copy()
        for d,atm in blist1:
            nsolid1.append(atm)
        nsolid2 = dats1.copy()
        for d,atm in blist2:
            nsolid2.append(atm)

        #Identify new number of atoms
        nnat1 = len(nsolid1)
        nnat2 = len(nsolid2)

        if nnat1 > nat1:
            ds = []
            #More atoms in new solid1 means solid2 has less atoms
            #Need to transfer atoms from solid1 to solid2
            #Find atoms that are too close
            for i in range(len(dist2)):
                for j in range(len(dist2),len(nsolid1)):
                    ds.append((nsolid1.get_distance(i,j),i))
            #Sort distances by longest to shortest
            ds = sorted(ds, key=lambda one:one[0])
            diff = nnat1-nat1
            exchangeindices = []
            while len(exchangeindices) < diff:
                #Grab shortest distance in ds list
                d, index = ds.pop()
                if index not in exchangeindices:
                    exchangeindices.append(index)
            nnsolid1 = Atoms()
            for i in range(len(nsolid1)):
                if i not in exchangeindices:
                    nnsolid1.append(nsolid1[i])
            nnsolid2 = nsolid2.copy()
            for i in exchangeindices:
                #for j in range(3):
                #	position[j]=nsolid2[i].position[j]-pt1[j]+pt2[j]
                position = [random.uniform(pt2[0]-r/(2**0.5),pt2[0]+r/(2**0.5)),random.uniform(pt2[1]-r/(2**0.5),pt2[1]+r/(2**0.5)),random.uniform(pt2[2]-r/(2**0.5),pt2[2]+r/(2**0.5))]
                nnsolid2.append(Atom(symbol=nsolid1[i].symbol, position=position))
            nsolid1 = nnsolid1.copy()
            nsolid2 = nnsolid2.copy()

        elif nnat1 < nat1:
            ds = []
            #More atoms in new solid2 means solid1 has less atoms
            #Need to transfer atoms from solid2 to solid1
            #Find atoms that are too close
            for i in range(len(dist1)):
                for j in range(len(dist1),len(nsolid2)):
                    ds.append((nsolid2.get_distance(i,j),i))
            #Sort distances by longest to shortest
            ds = sorted(ds, key=lambda one:one[0])
            diff = nnat2-nat2
            exchangeindices = []
            while len(exchangeindices) < diff:
                #Grab shortest distance in ds list
                d, index = ds.pop()
                if index not in exchangeindices:
                    exchangeindices.append(index)
            nnsolid2 = Atoms()
            for i in range(len(nsolid2)):
                if i not in exchangeindices:
                    nnsolid2.append(nsolid2[i])
            nnsolid1 = nsolid1.copy()
            for i in exchangeindices:
                #for j in range(3):
                #	position[j]=nsolid2[i].position[j]-pt1[j]+pt2[j]
                position = [random.uniform(pt1[0]-r/(2**0.5),pt1[0]+r/(2**0.5)),random.uniform(pt1[1]-r/(2**0.5),pt1[1]+r/(2**0.5)),random.uniform(pt1[2]-r/(2**0.5),pt1[2]+r/(2**0.5))]
                nnsolid1.append(Atom(symbol=nsolid2[i].symbol, position=position))
            nsolid1 = nnsolid1.copy()
            nsolid2 = nnsolid2.copy()

        #Number of atoms in each individual should now be preserved
        if Optimizer.forcing=='Concentration':
            #Need to check concentrations
            nconcent1 = []
            for one in symlist:
                atmss = [atm for atm in nsolid1 if atm.symbol==one]
                nconcent1.append(len(atmss))
            nconcent2 = []
            for one in symlist:
                atmss = [atm for atm in nsolid2 if atm.symbol==one]
                nconcent2.append(len(atmss))
            #Let's assume a random perturbation to concentration in order to correct this issue
            posd = []
            negd = []
            for i in range(len(nconcent1)):
                diff = nconcent1[i]-concent1[i]
                if diff >0:
                    posd.append((diff,symlist[i]))
                elif diff<0:
                    negd.append((diff,symlist[i]))
            for c,sym in negd:
                while c !=0:
                    symr = posd[0][1]
                    rlist = [atm.index for atm in nsolid1 if atm.symbol==symr]
                    index1 = random.choice(rlist)
                    nsolid1[index1].symbol=sym
                    c += 1
                    posd[0] = (posd[0][0]-1,posd[0][1])
                    if posd[0][0]==0:
                        posd = posd[1::]
            #Do the same for solid2
            posd = []
            negd = []
            for i in range(len(nconcent2)):
                diff = nconcent2[i]-concent2[i]
                if diff >0:
                    posd.append((diff,symlist[i]))
                elif diff<0:
                    negd.append((diff,symlist[i]))
            for c,sym in negd:
                while c !=0:
                    symr = posd[0][1]
                    rlist = [atm.index for atm in nsolid2 if atm.symbol==symr]
                    index1 = random.choice(rlist)
                    nsolid2[index1].symbol=sym
                    c += 1
                    posd[0] = (posd[0][0]-1,posd[0][1])
                    if posd[0][0]==0:
                        posd = posd[1::]

        #DEBUG: Write crossover to file
        if debug: 
            write_xyz(Optimizer.debugfile, nsolid1,'CX(randalloybx):nsolid1')
            write_xyz(Optimizer.debugfile, nsolid2,'CX(randalloybx):nsolid2')

        #DEBUG: Check structure of atoms exchanged
        for sym,c,m,u in Optimizer.atomlist:
                nc = len([atm for atm in nsolid1 if atm.symbol==sym])
                Optimizer.output.write('CX(randalloybx):New solid1 contains '+repr(nc)+' '+repr(sym)+' atoms\n')
                nc = len([atm for atm in nsolid2 if atm.symbol==sym])
                Optimizer.output.write('CX(randalloybx):New solid2 contains '+repr(nc)+' '+repr(sym)+' atoms\n')
        if Optimizer.forcing !='Concentration':
            for i in range(len(Optimizer.atomlist)):
                atms1 = [inds for inds in nsolid1 if inds.symbol==Optimizer.atomlist[i][0]]
                atms2 = [inds for inds in nsolid2 if inds.symbol==Optimizer.atomlist[i][0]]
                if len(atms1)==0:
                    if len(atms2)==0:
                        nsolid1[random.randint(0,len(indi1)-1)].symbol = Optimizer.atomlist[i][0]
                        nsolid2[random.randint(0,len(indi2)-1)].symbol = Optimizer.atomlist[i][0]
                    else:
                        nsolid1.append(atms2[random.randint(0,len(atms2)-1)])
                        nsolid1.pop(random.randint(0,len(nsolid1)-2))
                else:
                    if len(atms2)==0:
                        nsolid2.append(atms1[random.randint(0,len(atms1)-1)])
                        nsolid2.pop(random.randint(0,len(nsolid2)-2))	

        nsolid1.set_cell(cell1)
        nsolid2.set_cell(cell2)
        nsolid1.set_pbc(pbc1)
        nsolid2.set_pbc(pbc2)

        outs = find_defects(nsolid1,Optimizer.solidbulk,Optimizer.sf,atomlistcheck=Optimizer.atomlist,trackvacs=Optimizer.trackvacs,trackswaps=Optimizer.trackswaps,debug=False)
        ind1[0] = outs[0].copy()
        ind1.bulki = outs[1].copy()
        ind1.vacancies = outs[2].copy()
        ind1.swaps = outs[3].copy()
        outs = find_defects(nsolid2,Optimizer.solidbulk,Optimizer.sf,atomlistcheck=Optimizer.atomlist,trackvacs=Optimizer.trackvacs,trackswaps=Optimizer.trackswaps,debug=False)
        ind2[0] = outs[0].copy()
        ind2.bulki = outs[1].copy()
        ind2.vacancies = outs[2].copy()
        ind2.swaps = outs[3].copy()
    
    return ind1, ind2



import numpy
import copy
import random
from MAST.structopt.switches import fitness_switch
from MAST.structopt.moves.lattice_alteration_group import lattice_alteration_group
from MAST.structopt.moves.lattice_alteration_rdrd import lattice_alteration_rdrd

def basin_hop_la(indiv, Optimizer):
    """Move function to perform mini-basin hopping run to Translate atoms
    Inputs:
        indiv = Individual class object to be altered
        Optimizer = Optimizer class object with needed parameters
    Outputs:
        indiv = Altered Individual class object
    *** needs work ***
    """
    if 'MU' in Optimizer.debug:
        debug = True
    else:
        debug = False
    Optimizer.output.write('Performing Basin Hopping Lattice Alteration Mutation on indiv '+repr(indiv.index)+'\n')
    #Save copy of starting positions
    startindiv=indiv[0].copy()
    startbulk=indiv.bulki
    indivmark=len(startindiv)
    alloysetting=Optimizer.alloy
    fingerprintsetting=Optimizer.fingerprinting
    if Optimizer.structure=='Defect':
        finddefectsetting=Optimizer.finddefects
    mutation_options = copy.deepcopy(Optimizer.mutation_options)
    startindiv = indiv.duplicate()
    
    #Get starting fitness
    fitmin=indiv.fitness
    if indiv.fitness==0:
        out=fitness_switch([Optimizer,indiv])
        fitmin=out[0].fitness
    options = ['lattice_alteration_group', 'lattice_alteration_rdrd']
    
    totalsteps=Optimizer.bh_steps
    kt=Optimizer.bh_temp
    flag=False
    for step in range(totalsteps):
        prevind = indiv.duplicate()
        scheme = random.choice(options)
        indiv = eval(scheme+'(indiv, Optimizer)')
        out=fitness_switch([Optimizer,indiv])
        fitnew=out[0].fitness
        if fitnew < fitmin:
            flag=True
            break
        else:
            accept=numpy.exp((fitmin - fitnew) / kt) > random.random()
            if not accept:
                indiv = prevind

    Optimizer.output.write('Evaluated '+repr(step)+' steps\n')
    if flag==False:
        Optimizer.output.write('Failed to find lower energy lattice altered structure\n')
        #indiv = startindiv
    else:
        Optimizer.output.write('Found lower energy lattice altered structure\n')
    Optimizer.alloy=alloysetting
    Optimizer.fingerprinting=fingerprintsetting
    if Optimizer.structure=='Defect':
        Optimizer.finddefects=finddefectsetting
    Optimizer.mutation_options = mutation_options
    muttype='BHLA'+repr(step)
    if indiv.energy==0:
        indiv.history_index=indiv.history_index+'m'+muttype
    else:
        indiv.history_index=repr(indiv.index)+'m'+muttype
    return indiv
		
		


import random
import numpy
from ase import Atom, Atoms
from MAST.structopt.inp_out import write_xyz

def clustbx(ind1, ind2, Optimizer):
    """Select a box in the cluster configuration
    """
    if 'CX' in Optimizer.debug:
        debug = True
    else:
        debug = False
    Optimizer.output.write('Box Cluster Cx between individual '+repr(ind1.index)+' and individual '+repr(ind2.index)+'\n')
    
    #Perserve starting conditions of individual
    solid1=ind1[0].copy()
    solid2=ind2[0].copy()
    cello1=ind1[0].get_cell()
    cello2=ind2[0].get_cell()
    cell1=numpy.maximum.reduce(solid1.get_positions())#-numpy.minimum.reduce(solid1.get_positions())
    cell2=numpy.maximum.reduce(solid2.get_positions())#-numpy.minimum.reduce(solid2.get_positions())
    cell=numpy.minimum(cell1,cell2)
    pbc1=solid1.get_pbc()
    pbc2=solid2.get_pbc()

    #Get starting concentrations and number of atoms
    nat1=len(solid1)
    nat2=len(solid2)
    
    if Optimizer.alloy==False:
        #Atomlist-based
        concent1=[c for sym,c,m,u in Optimizer.atomlist]
        concent2=[c for sym,c,m,u in Optimizer.atomlist]
        symlist=[sym for sym,c,m,u in Optimizer.atomlist]
    else:
        symlist=list(set(solid1.get_chemical_symbols()))
        #Assumes same types of atoms in both solid1 and solid2
        concent1=[]
        for one in symlist:
            atmss=[atm for atm in solid1 if atm.symbol==one]
            concent1.append(len(atmss))
        concent2=[]
        for one in symlist:
            atmss=[atm for atm in solid2 if atm.symbol==one]
            concent2.append(len(atmss))

    # Pick a origin point for box in the cell
    #pt=[random.uniform(0,cell[0]),random.uniform(0,cell[1]),random.uniform(0,cell[2])]
    pt1=random.choice(solid1).position
    pt2=random.choice(solid2).position

    #Find max radius of circle cut
    r1=min([min(cell1[0]-pt1[0],pt1[0]),min(cell1[1]-pt1[1],pt1[1]),min(cell1[2]-pt1[2],pt1[2])])
    r2=min([min(cell2[0]-pt2[0],pt2[0]),min(cell2[1]-pt2[1],pt2[1]),min(cell2[2]-pt2[2],pt2[2])])
    r=min(r1,r2)
    mcell=min(cell)
    if r > mcell*0.125:
        r=mcell*0.125
    elif r==0.0:
        r=1.0
    if debug:
        print 'Radius of box = '+repr(r)+'\nPosition in solid1 = '+repr(pt1)+'\nPosition in solid2 = '+repr(pt2)
    
    #Find atoms within sphere of radius r
    solid1.append(Atom(position=pt1))
    dist1=[]
    blist1=[]
    for i in range(len(solid1)-1):
        d=solid1.get_distance(i,len(solid1)-1)
        if d < r:
            dist1.append((d,solid1[i]))
        else:
            blist1.append((d,solid1[i]))
    solid1.pop()

    solid2.append(Atom(position=pt2))
    dist2=[]
    blist2=[]
    for i in range(len(solid2)-1):
        d=solid2.get_distance(i,len(solid2)-1)
        if d < r:
            dist2.append((d,solid2[i]))
        else:
            blist2.append((d,solid2[i]))
    solid2.pop()
    
    #Translate spheres to opposite location
    dats1=Atoms()
    for d,atm in dist1:
        dats1.append(atm)
    dats1.translate(-pt1)
    dats1.translate(pt2)
    dats2=Atoms()
    for d,atm in dist2:
        dats2.append(atm)
    dats2.translate(-pt2)
    dats2.translate(pt1)

    #Exchange atoms in sphere and build new solids
    nsolid1=dats2.copy()
    for d,atm in blist1:
        nsolid1.append(atm)
    nsolid2=dats1.copy()
    for d,atm in blist2:
        nsolid2.append(atm)

    #Identify new number of atoms
    nnat1=len(nsolid1)
    nnat2=len(nsolid2)

    if nnat1 > nat1:
        ds=[]
        #More atoms in new solid1 means solid2 has less atoms
        #Need to transfer atoms from solid1 to solid2
        #Find atoms that are too close
        for i in range(len(dist2)):
            for j in range(len(dist2),len(nsolid1)):
                ds.append((nsolid1.get_distance(i,j),i))
        #Sort distances by longest to shortest
        ds=sorted(ds, key=lambda one:one[0])
        diff=nnat1-nat1
        exchangeindices=[]
        while len(exchangeindices) < diff:
            #Grab shortest distance in ds list
            d, index=ds.pop()
            if index not in exchangeindices:
                exchangeindices.append(index)
        nnsolid1=Atoms()
        for i in range(len(nsolid1)):
            if i not in exchangeindices:
                nnsolid1.append(nsolid1[i])
        nnsolid2=nsolid2.copy()
        for i in exchangeindices:
            #for j in range(3):
            #	position[j]=nsolid2[i].position[j]-pt1[j]+pt2[j]
            position=[random.uniform(pt2[0]-r/(2**0.5),pt2[0]+r/(2**0.5)),random.uniform(pt2[1]-r/(2**0.5),pt2[1]+r/(2**0.5)),random.uniform(pt2[2]-r/(2**0.5),pt2[2]+r/(2**0.5))]
            nnsolid2.append(Atom(symbol=nsolid1[i].symbol, position=position))
        nsolid1=nnsolid1.copy()
        nsolid2=nnsolid2.copy()

    elif nnat1 < nat1:
        ds=[]
        #More atoms in new solid2 means solid1 has less atoms
        #Need to transfer atoms from solid2 to solid1
        #Find atoms that are too close
        for i in range(len(dist1)):
            for j in range(len(dist1),len(nsolid2)):
                ds.append((nsolid2.get_distance(i,j),i))
        #Sort distances by longest to shortest
        ds=sorted(ds, key=lambda one:one[0])
        diff=nnat2-nat2
        exchangeindices=[]
        while len(exchangeindices) < diff:
            #Grab shortest distance in ds list
            d, index=ds.pop()
            if index not in exchangeindices:
                exchangeindices.append(index)
        nnsolid2=Atoms()
        for i in range(len(nsolid2)):
            if i not in exchangeindices:
                nnsolid2.append(nsolid2[i])
        nnsolid1=nsolid1.copy()
        for i in exchangeindices:
            #for j in range(3):
            #	position[j]=nsolid2[i].position[j]-pt1[j]+pt2[j]
            position=[random.uniform(pt1[0]-r/(2**0.5),pt1[0]+r/(2**0.5)),random.uniform(pt1[1]-r/(2**0.5),pt1[1]+r/(2**0.5)),random.uniform(pt1[2]-r/(2**0.5),pt1[2]+r/(2**0.5))]
            nnsolid1.append(Atom(symbol=nsolid2[i].symbol, position=position))
        nsolid1=nnsolid1.copy()
        nsolid2=nnsolid2.copy()

    #Number of atoms in each individual should now be preserved
    try:
        if Optimizer.forcing=='Concentration':
            #Need to check concentrations
            nconcent1=[]
            for one in symlist:
                atmss=[atm for atm in nsolid1 if atm.symbol==one]
                nconcent1.append(len(atmss))
            nconcent2=[]
            for one in symlist:
                atmss=[atm for atm in nsolid2 if atm.symbol==one]
                nconcent2.append(len(atmss))
            #Let's assume a random perturbation to concentration in order to correct this issue
            posd=[]
            negd=[]
            for i in range(len(nconcent1)):
                diff=nconcent1[i]-concent1[i]
                if diff >0:
                    posd.append((diff,symlist[i]))
                elif diff<0:
                    negd.append((diff,symlist[i]))
            for c,sym in negd:
                while c !=0:
                    symr=posd[0][1]
                    rlist=[atm.index for atm in nsolid1 if atm.symbol==symr]
                    index1=random.choice(rlist)
                    nsolid1[index1].symbol=sym
                    c+=1
                    posd[0]=(posd[0][0]-1,posd[0][1])
                    if posd[0][0]==0:
                        posd=posd[1::]
            #Do the same for solid2
            posd=[]
            negd=[]
            for i in range(len(nconcent2)):
                diff=nconcent2[i]-concent2[i]
                if diff >0:
                    posd.append((diff,symlist[i]))
                elif diff<0:
                    negd.append((diff,symlist[i]))
            for c,sym in negd:
                while c !=0:
                    symr=posd[0][1]
                    rlist=[atm.index for atm in nsolid2 if atm.symbol==symr]
                    index1=random.choice(rlist)
                    nsolid2[index1].symbol=sym
                    c+=1
                    posd[0]=(posd[0][0]-1,posd[0][1])
                    if posd[0][0]==0:
                        posd=posd[1::]
    except:
        f=open('problem-structures.xyz','a')
        write_xyz(f,nsolid1,data='Failed - CX(randalloybx):nsolid1 '+ind1.history_index)
        write_xyz(f,nsolid2,data='Failed - CX(randalloybx):nsolid2 '+ind2.history_index)
        nsolid1=ind1[0].copy()
        nsolid2=ind2[0].copy()
        f.close()
    #DEBUG: Write crossover to file
    if debug: 
        write_xyz(Optimizer.debugfile, nsolid1,'CX(randalloybx):nsolid1')
        write_xyz(Optimizer.debugfile, nsolid2,'CX(randalloybx):nsolid2')


    #DEBUG: Check structure of atoms exchanged
    for sym,c,m,u in Optimizer.atomlist:
            nc=len([atm for atm in nsolid1 if atm.symbol==sym])
            Optimizer.output.write('CX(clustbx):New solid1 contains '+repr(nc)+' '+repr(sym)+' atoms\n')
            nc=len([atm for atm in nsolid2 if atm.symbol==sym])
            Optimizer.output.write('CX(clustbx):New solid2 contains '+repr(nc)+' '+repr(sym)+' atoms\n')
    if Optimizer.forcing !='Concentration':
        for i in range(len(Optimizer.atomlist)):
            atms1=[inds for inds in nsolid1 if inds.symbol==Optimizer.atomlist[i][0]]
            atms2=[inds for inds in nsolid2 if inds.symbol==Optimizer.atomlist[i][0]]
            if len(atms1)==0:
                if len(atms2)==0:
                    nsolid1[random.randint(0,len(indi1)-1)].symbol==Optimizer.atomlist[i][0]
                    nsolid2[random.randint(0,len(indi2)-1)].symbol==Optimizer.atomlist[i][0]
                else:
                    nsolid1.append(atms2[random.randint(0,len(atms2)-1)])
                    nsolid1.pop(random.randint(0,len(nsolid1)-2))
            else:
                if len(atms2)==0:
                    nsolid2.append(atms1[random.randint(0,len(atms1)-1)])
                    nsolid2.pop(random.randint(0,len(nsolid2)-2))	

    nsolid1.set_cell(cello1)
    nsolid2.set_cell(cello2)
    nsolid1.set_pbc(pbc1)
    nsolid2.set_pbc(pbc2)
    
    ind1[0]=nsolid1.copy()
    ind2[0]=nsolid2.copy()

    return ind1, ind2


import os
import shutil
import sys
import time
import logging
import copy
import subprocess
from MAST.utility import loggerutils
from MAST.utility import MASTFile
from MAST.utility import dirutil
from MAST.utility import MASTError
from MAST.ingredients.checker import BaseChecker
from MAST.ingredients.checker import VaspChecker
from MAST.ingredients.checker import LammpsChecker
from MAST.submit import queue_commands
from pymatgen.core.structure import Structure
from pymatgen.io.aseio import AseAtomsAdaptor
from MAST.structopt.Optimizer import Optimizer
from MAST.structopt import post_processing as pp
from MAST.structopt import inp_out
from MAST.structopt.switches import fitness_switch

class StructoptChecker(BaseChecker):
    
    def __init__(self, **kwargs):
        """Please not modify this init method."""
        allowed_keys = {
            'name' : (str, str(), 'Name of directory'),
            'program': (str, str(), 'Program, e.g. "vasp"'),
            'program_keys': (dict, dict(), 'Dictionary of program keywords'),
            'structure': (Structure, None, 'Pymatgen Structure object')
            }
        BaseChecker.__init__(self, allowed_keys, **kwargs)
        #print program_keys
        self.structopt_parameters = self._structopt_get_non_mast_keywords()
        self.potential_file = None
        if 'potential_file' in self.structopt_parameters:
            if self.structopt_parameters['potential_file'] != 'None':
                if os.path.isfile(self.structopt_parameters['potential_file']):
                    self.potential_file = self.structopt_parameters['potential_file']
                else:
                    raise MASTError(self.__class__.__name__,
                        "Cannot find specified potential file for LAMMPS: %s" % self.structopt_parameters['potential_file'])
        if "MAST" in self.structopt_parameters['calc_method']:
            if "exec_mast" not in self.structopt_parameters:
                raise MASTError(self.__class__.__name__,
                    "Must specify command for Mast execution in structopt. Use exec_mast command")

    def _structopt_get_non_mast_keywords(self):
        """Get the StructOpt keywords and make a dictionary."""
        input_dict=dict()
        allowedpath = os.path.join(dirutil.get_mast_control_path(),
                        'programkeys','structopt_allowed_keywords.py')
        allowed_list = self._structopt_get_allowed_keywords(allowedpath)
        for key, value in self.keywords['program_keys'].iteritems():
            if not key[0:5] == "mast_":
                keytry = key
                if not (keytry in allowed_list):
                    self.logger.warning("Ignoring program key %s for INPUT. To allow this keyword, add it to %s" % (keytry, allowedpath))
                else:
                    try:
                        #Convert numbers to floats
                        value = float(value)
                    except:
                        try:
                            #Convert list, tuple, and booleans
                            if value != 'calc_method':
                                value = eval(value)
                            else:
                                value = value.strip()
                        except:
                            try:
                                #Leave remaining as strings
                                value = value.strip()
                            except:
                                print 'Trouble with input line: ', value
                    input_dict[keytry]=value
        if 'structopt_input_file' in input_dict:
            self.logger.info('Requested read from StructOpt input file at : {0}'.format(input_dict['structopt_input_file']))
            new_dict = inp_out.read_parameter_input(input_dict['structopt_input_file'],False)
            #Overwrite parameters from input file with parameters in mast input
            for key, value in input_dict.iteritems():
                new_dict[key] = value
            input_dict = new_dict
            self.logger.info('New input_dict as read from StructOpt input file : {0}'.format(input_dict))
        # Check for potential file
        if 'potential_file' in input_dict:
            if 'pot_file' not in input_dict:
                input_dict['pot_file']
        return input_dict

    def _structopt_get_allowed_keywords(self, allowedpath):
        """Get allowed vasp keywords.
            Args:
                allowedpath <str>: file path for allowed lammps keywords
        """
        allowed = MASTFile(allowedpath)
        allowed_list=list()
        for entry in allowed.data:
            allowed_list.append(entry.strip())
        return allowed_list
    
    def transfer_restart_files(self, input_dict):
        if 'filename' in input_dict:
            fname = '{0}-rank0'.format(input_dict['filename'])
        else:
            fname = 'Output-rank0'
        if ('restart' in input_dict) and (input_dict['restart']):
            try:
                restartdir = os.path.join(self.keywords['name'],fname)
                files = os.listdir(restartdir)
            except:
                raise MASTError(self.__class__.__name__,
                    "Cannot find directory for restart: {0}".format(restartdir))
            return input_dict
        if input_dict['restart_optimizer']:
            if 'ARCHIVE' in input_dict['structopt_input_file']:
                replaceflag = True
            else:
                replaceflag = False
            path = os.path.join(self.keywords['name'],fname)
            if not os.path.exists(path):
                os.mkdir(path)
            outputoptions = ['optimizerfile', 'tenergyfile', 'fpfile',
                            'fpminfile', 'debugfile', 'Genealogyfile', 'summary']
            for one in outputoptions:
                if one in input_dict:
                    if (input_dict[one] != None):
                        bname = os.path.basename(input_dict[one])
                        npath = os.path.join(path,bname)
                        if replaceflag:
                            input_dict[one] = input_dict[one].replace('SCRATCH','ARCHIVE')
                        shutil.copyfile(input_dict[one], npath)
                        input_dict[one] = npath
            flist = list()
            for one in input_dict['files']:
                bname = os.path.basename(one)
                npath = os.path.join(path,bname)
                if replaceflag:
                    one = one.replace('SCRATCH','ARCHIVE')
                shutil.copyfile(one, npath)
                flist.append(npath)
            input_dict['files'] = flist
            iflist = list()
            if input_dict['ifiles']:
                for one in input_dict['ifiles']:
                    bname = os.path.basename(one)
                    npath = os.path.join(path,bname)
                    if replaceflag:
                        one = one.replace('SCRATCH','ARCHIVE')
                    shutil.copyfile(one, npath)
                    iflist.append(npath)
            input_dict['ifiles'] = iflist
            bname = os.path.basename(input_dict['output'])
            npath = os.path.join(self.keywords['name'],bname)
            if replaceflag:
                input_dict['output'] = input_dict['output'].replace('SCRATCH','ARCHIVE')
            fout = open(input_dict['output'],'r')
            lines = fout.readlines()
            fout.close()
            nlines = []
            for line in lines:
                if "End of Execution" not in line:
                    nlines.append(line)
            foutn = open(npath,'a')
            for line in nlines:
                foutn.write(line)
            foutn.close()
            #shutil.copyfile(input_dict['output'], npath)
            input_dict['output'] = npath
            path = os.path.join(path, 'Restart-files')
            if not os.path.exists(path):
                os.mkdir(path)
            popfiles = list()
            for one in input_dict['population']:
                bname = os.path.basename(one)
                npath = os.path.join(path,bname)
                if replaceflag:
                    one = one.replace('SCRATCH','ARCHIVE')
                shutil.copy(one, npath)
                popfiles.append(npath)
            input_dict['population'] = popfiles
            bestfiles = list()
            for one in input_dict['BESTS']:
                bname = os.path.basename(one)
                npath = os.path.join(path,bname)
                if replaceflag:
                    one = one.replace('SCRATCH','ARCHIVE')
                shutil.copy(one, npath)
                bestfiles.append(npath)
            input_dict['BESTS'] = bestfiles
            return input_dict
    
    def is_frozen(self):
        """Check if StructOpt calculation is frozen.
        """
        if 'filename' in self.structopt_parameters:
            outfile = self.structopt_parameters['filename']
        else:
            outfile = "Output.txt"
        if not os.path.isfile(os.path.join(self.keywords['name'], outfile)):
            return False
        return BaseChecker.is_frozen(self,outfile)
    
    def is_complete(self):
        if "MAST" in self.structopt_parameters['calc_method']:
            if "LOOPED" in self.structopt_parameters['calc_method']:
                return self.is_complete_looped()
            else:
                return self.is_complete_one_gen()
        else:
            return self.is_complete_whole()
        
    def is_complete_whole(self):
        """Check if StructOpt calculation is complete.
        """
        if 'filename' in self.structopt_parameters:
            fname = '{0}-rank0.txt'.format(self.structopt_parameters['filename'])
            opath = os.path.join(self.keywords['name'],fname)
        else:
            opath = os.path.join(self.keywords['name'],'Output-rank0.txt')
        if not os.path.isfile(opath):
            self.logger.info("No StructOpt output at %s; not complete." % opath)
            return False
        reachgrep=subprocess.Popen('grep "End of Execution" %s' % opath, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        reachrpt=reachgrep.communicate()[0]
        reachgrep.wait()
        if reachrpt=='':
            self.logger.info("StructOpt output file at %s shows calculation is incomplete." % opath)
            return False
        else:
            self.logger.info("StructOpt output file at %s shows calculation has finished." % opath)
            return True
    
    def is_complete_one_gen(self):
        """Check if StructOpt single generation is complete.
        """
        if 'filename' in self.structopt_parameters:
            opath = os.path.join(self.keywords['name'],self.structopt_parameters['filename'])
        else:
            opath = os.path.join(self.keywords['name'],'Output-rank0.txt')
#         if not os.path.isfile(opath):
#             self.logger.info("No StructOpt output at %s; not complete." % opath)
#             return False
        reachgrep=subprocess.Popen('grep "End of Execution" %s' % opath, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        reachrpt=reachgrep.communicate()[0]
        reachgrep.wait()
        if reachrpt=='':
            reachedend=False
        else:
            self.logger.info("StructOpt output file at %s shows calculation has finished." % opath)
            return True
        if 'filename' in self.structopt_parameters:
            optifile = os.path.join(os.path.join(self.keywords['name'], '{0}-rank0'.format(self.structopt_parameters['filename']),'Optimizer-restart-file.txt'))
        else:
            optifile = os.path.join(os.path.join(self.keywords['name'], 'Output-rank0','Optimizer-restart-file.txt'))
        ingpath = self.keywords['name']
        if not os.path.isfile(optifile):
            os.chdir(ingpath)
            inputfile = os.path.join(ingpath,"structoptinput.txt")
            Opti = Optimizer(inputfile)
            Opti.algorithm_initialize()
            self.logger.info("Opti initialized")
            self.logger.info('Creating files for initial population')
        else:
            if not self.subfolders_complete():
                self.logger.info("Subfolders not complete. No further checks.")
                self.flip_status_back()
                return False
            else: 
                self.logger.info("Evaluating completed structures.")
                Opti = Optimizer(input=optifile)
                self.logger.info('Optimizer read optimizer restart file')
                self.logger.info("Generation %i" % Opti.generation)
                Opti = self.evaluate_structures_for_fitness(Opti)
                if Opti.convergence:
                    os.chdir(ingpath)
                    end_signal = Opti.algorithm_stats(Opti.population)
                    cwd = os.getcwd()
                    if Opti.postprocessing:
                        self.logger.info('Running Post-processing')
                        path = os.path.join(os.getcwd(), '{0}-rank{1}'.format(Opti.filename,0))
                        os.chdir(path)
                        if Opti.genealogytree:
                            pp.read_output(os.getcwd(),genealogytree=True,natoms=Opti.natoms)
                        else:
                            pp.read_output(os.getcwd(),genealogytree=False,natoms=Opti.natoms)
                        os.chdir(cwd)
                    if Opti.lattice_concentration:
                        if Opti.structure=='Defect':
                            self.logger.info('Running lattice concentration check')
                            path = os.path.join(os.getcwd(), '{0}-rank{1}'.format(Opti.filename,rank))
                            os.chdir(path)
                            if Opti.BestIndsList:
                                pp.get_lattice_concentration(os.path.join(os.getcwd(),'Bulkfile.xyz'),os.path.join(os.getcwd(),'Bests-'+Opti.filename+'.xyz'))
                            else:
                                pp.get_lattice_concentration(os.path.join(os.getcwd(),'Bulkfile.xyz'),os.path.join(os.getcwd(),'indiv00.xyz'))
                            os.chdir(cwd)
                    Opti.write()
                    return True
        if Opti.generation !=0:
            self.logger.info("Mutating and evolving structures for generation %i" % Opti.generation)
        else:
            self.logger.info('Creating files for initial population')
        self.mutate_evolve_write_new_structures(Opti)
        self.logger.info("Changing status back.")
        self.flip_status_back()
        os.chdir(ingpath)
        Opti.write()
        return False
    
    def is_complete_looped(self):
        """Check if StructOpt looped generations are complete.
        """
        if 'filename' in self.structopt_parameters:
            opath = os.path.join(self.keywords['name'],self.structopt_parameters['filename'])
        else:
            opath = os.path.join(self.keywords['name'],'Output-rank0.txt')
        reachgrep=subprocess.Popen('grep "End of Execution" %s' % opath, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        reachrpt=reachgrep.communicate()[0]
        reachgrep.wait()
        if reachrpt=='':
            reachedend=False
        else:
            self.logger.info("StructOpt output file at %s shows calculation has finished." % opath)
            return True
        if 'filename' in self.structopt_parameters:
            optifile = os.path.join(os.path.join(self.keywords['name'], '{0}-rank0'.format(self.structopt_parameters['filename']),'Optimizer-restart-file.txt'))
        else:
            optifile = os.path.join(os.path.join(self.keywords['name'], 'Output-rank0','Optimizer-restart-file.txt'))
        ingpath = copy.deepcopy(self.keywords['name'])
        count = 0
        pwd = os.getcwd()
        while count < 10:
            if not os.path.isfile(optifile):
                os.chdir(ingpath)
                inputfile = os.path.join(ingpath,"structoptinput.txt")
                Opti = Optimizer(inputfile)
                Opti.algorithm_initialize()
                self.logger.info("Opti initialized")
                self.logger.info('Creating files for initial population')
                completeflag=True
            else:
                if not self.subfolders_complete_looped():
                    self.logger.info("Subfolders not complete. No further checks.")
                    self.flip_status_back()
                    completeflag = False
                else: 
                    self.logger.info("Evaluating completed structures.")
                    Opti = Optimizer(input=optifile)
                    self.logger.info('Optimizer read optimizer restart file')
                    self.logger.info("Generation %i" % Opti.generation)
                    Opti = self.evaluate_structures_for_fitness(Opti)
                    completeflag=True
                    if Opti.convergence:
                        os.chdir(ingpath)
                        end_signal = Opti.algorithm_stats(Opti.population)
                        if Opti.postprocessing:
                            self.logger.info('Running Post-processing')
                            path = os.path.join(os.getcwd(), '{0}-rank{1}'.format(Opti.filename,0))
                            os.chdir(path)
                            if Opti.genealogytree:
                                pp.read_output(os.getcwd(),genealogytree=True,natoms=self.natoms)
                            else:
                                pp.read_output(os.getcwd(),genealogytree=False,natoms=self.natoms)
                            os.chdir(cwd)
                        if Opti.lattice_concentration:
                            if Opti.structure=='Defect':
                                self.logger.info('Running lattice concentration check')
                                path = os.path.join(os.getcwd(), '{0}-rank{1}'.format(Opti.filename,rank))
                                os.chdir(path)
                                if Opti.BestIndsList:
                                    pp.get_lattice_concentration(os.path.join(os.getcwd(),'Bulkfile.xyz'),os.path.join(os.getcwd(),'Bests-'+Opti.filename+'.xyz'))
                                else:
                                    pp.get_lattice_concentration(os.path.join(os.getcwd(),'Bulkfile.xyz'),os.path.join(os.getcwd(),'indiv00.xyz'))
                                os.chdir(cwd)
                        Opti.write()
                        return True
            if completeflag:
                if Opti.generation !=0:
                    self.logger.info("Mutating and evolving structures for generation %i" % Opti.generation)
                else:
                    self.logger.info('Creating files for initial population')
                self.mutate_evolve_write_new_structures(Opti)
                self.logger.info("Changing status back.")
                self.flip_status_back()
                os.chdir(ingpath)
                Opti.write()
            print "Submitting from submission list."
            self.submit_from_submission_list()
            print "Clearing submission list."
            queue_commands.clear_submission_list()
#             mycwd=os.getcwd()
#             mast_control = os.getenv("MAST_CONTROL")
#             os.chdir(mast_control)
#             mycommand=queue_commands.queue_submission_command("mastmon_submit.sh") #run the mastmon_submit.sh script in $MAST_CONTROL which creates a mastmon in order to do status checking on a compute node
#             mysub = subprocess.Popen(mycommand, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
#             mysub.wait()
#             os.chdir(mycwd)
            os.chdir(pwd)
            self.keywords['name'] = ingpath
            count +=1
            time.sleep(60)
        return False
    
    def is_ready_to_run(self):
        """Check if StructOpt ingredient is 
            ready to run.
        """
        dirname = self.keywords['name']
        flist = os.listdir(dirname)
        notready = 0
        if "structoptinput.txt" not in flist:
            notready += 1
        if self.potential_file:
            potfile = os.path.basename(self.potential_file)
            if potfile not in flist:
                notready += 1
        if "submit.sh" not in flist:
            notready = notready + 1
        if notready > 0:
            return False
        else:
            import sys
            ingpath = self.keywords['name']
            inputfile = os.path.join(ingpath,"structoptinput.txt")
            exec_line = self.keywords['program_keys']['mast_exec']
            sp_exec_line = exec_line.split(' ')
            new_exec_line = str()
            for segment in sp_exec_line:
                if ('.py' not in segment):
                    new_exec_line += segment + ' '
            try:
                optimizerpath = os.path.join(os.environ['MAST_CONTROL'],'Optimizer.py')
            except:
                raise MASTError(self.__class__.__name__,
                    "Cannot find Optimizer run file in 'structopt'! Current path: {0}".format(os.path.join(os.environ['MAST_CONTROL'],'Optimizer.py')))
            new_exec_line += ' {0} {1}'.format(optimizerpath,inputfile)
            self.keywords['program_keys']['mast_exec'] = new_exec_line
            return True
    
    def is_ready_to_next(self):
        if not self.subfolders_complete():
            self.logger.info("Subfolders not complete. No further checks.")
            self.flip_status_back()
            return False
        else:
            return True
    
    def is_started(self):
        """See if the ingredient has been started on
            the queue.
        """
        if os.path.isfile(os.path.join(self.keywords['name'],'structoptinput.txt')):
            return True
        else:
            return False
    
    def set_up_program_input(self):
        input_dict = self.structopt_parameters
        if ('restart_optimizer' in input_dict) and (input_dict['restart_optimizer']==True):
            if ('structopt_input_file' in input_dict) and \
                ('Optimizer-restart-file' in input_dict['structopt_input_file']):
                input_dict = self.transfer_restart_files(input_dict)
        if ('restart' in input_dict) and (input_dict['restart']==True):
            input_dict = self.transfer_restart_files(input_dict)
        self.structopt_parameters = input_dict
        if "MAST" in self.structopt_parameters['calc_method']:
            if 'LOOPED' in self.structopt_parameters['calc_method']:
                return self.set_up_program_input_looped()
            else:
                return self.set_up_program_input_one_gen()
        else:
            return self.set_up_program_input_whole()
    
    def set_up_program_input_whole(self):
        """Set up the StructOpt input file."""
        self.logger.info('Parameters for structopt: {0}'.format(self.structopt_parameters))
        if ('restart_optimizer' in self.structopt_parameters and self.structopt_parameters['restart_optimizer']):
            population_list = self.structopt_parameters['population']
            bests_list = self.structopt_parameters['BESTS']
        Optinit = Optimizer(self.structopt_parameters)
        ingpath = self.keywords['name']
        inputfile = os.path.join(ingpath,"structoptinput.txt")
        if 'potential_file' in self.structopt_parameters:
            newlocation = os.path.join(ingpath,os.path.basename(self.structopt_parameters['potential_file']))
            shutil.copyfile(self.structopt_parameters['potential_file'],newlocation)
            if not Optinit.pot_file:
                Optinit.pot_file = os.path.basename(self.structopt_parameters['potential_file'])
        if Optinit.restart_optimizer:
            Optinit.population = population_list
            Optinit.BESTS = bests_list
            Optinit.write(inputfile)
        else:
            Optinit.write(inputfile,restart=False)
        import sys
        exec_line = self.keywords['program_keys']['mast_exec']
        sp_exec_line = exec_line.split(' ')
        new_exec_line = str()
        for segment in sp_exec_line:
            if ('.py' not in segment):
                new_exec_line += segment + ' '
        try:
            structoptpath = os.path.join(os.environ['MAST_CONTROL'],'Optimizer.py')
        except:
            raise MASTError(self.__class__.__name__,
                "Cannot find Optimizer run file in 'structopt'! Current path: {0}".format(os.path.join(os.environ['MAST_CONTROL'],'Optimizer.py')))
        new_exec_line += ' {0} {1}'.format(structoptpath,inputfile)
        self.keywords['program_keys']['mast_exec'] = new_exec_line
        return
    
    def set_up_program_input_one_gen(self):
        """Set up the StructOpt input file."""
        Optinit = Optimizer(self.structopt_parameters)
        ingpath = self.keywords['name']
        inputfile = os.path.join(ingpath,"structoptinput.txt")
        if 'potential_file' in self.structopt_parameters:
            newlocation = os.path.join(ingpath,os.path.basename(self.structopt_parameters['potential_file']))
            shutil.copyfile(self.structopt_parameters['potential_file'],newlocation)
            if not Optinit.pot_file:
                Optinit.pot_file = os.path.basename(self.structopt_parameters['potential_file'])
        Optinit.write(inputfile,restart=False)
        self.keywords['program_keys']['mast_exec'] = "mast"
        return
    
    def set_up_program_input_looped(self):
        """Set up the StructOpt input file."""
        Optinit = Optimizer(self.structopt_parameters)
        ingpath = self.keywords['name']
        inputfile = os.path.join(ingpath,"structoptinput.txt")
        if 'potential_file' in self.structopt_parameters:
            newlocation = os.path.join(ingpath,os.path.basename(self.structopt_parameters['potential_file']))
            shutil.copyfile(self.structopt_parameters['potential_file'],newlocation)
            if not Optinit.pot_file:
                Optinit.pot_file = os.path.basename(self.structopt_parameters['potential_file'])
        Optinit.write(inputfile,restart=False)
        mast_control = os.getenv("MAST_CONTROL")
        runmastpath = os.path.join(mast_control,"runmast.py")
        #mycommand=queue_commands.queue_submission_command("mastmon_submit.sh") 
        #"./"+os.path.join(mast_control,"mastmon_submit.sh")
        self.keywords['program_keys']['mast_exec'] = "python {0} >> $MAST_CONTROL/mastoutput 2> $MAST_CONTROL/errormast".format(runmastpath)
        return
      
    def eval_generation(self,optifile):
        self.logger.info("Evaluating completed structures.")
        Opti = Optimizer(input=optifile)
        self.logger.info('Optimizer read optimizer restart file')
        self.logger.info("Generation %i" % Opti.generation)
        Opti = self.evaluate_structures_for_fitness(Opti)
        #Check for convergence and run stats and postprocessing as desired if True
        if Opti.convergence:
            end_signal = Opti.algorithm_stats(Opti.population)
            if Opti.postprocessing:
                self.logger.info('Running Post-processing')
                path = os.path.join(os.getcwd(), '{0}-rank{1}'.format(Opti.filename,0))
                os.chdir(path)
                if Opti.genealogytree:
                    pp.read_output(os.getcwd(),genealogytree=True,natoms=self.natoms)
                else:
                    pp.read_output(os.getcwd(),genealogytree=False,natoms=self.natoms)
                os.chdir(cwd)
            if Opti.lattice_concentration:
                if Opti.structure=='Defect':
                    self.logger.info('Running lattice concentration check')
                    path = os.path.join(os.getcwd(), '{0}-rank{1}'.format(Opti.filename,rank))
                    os.chdir(path)
                    if Opti.BestIndsList:
                        pp.get_lattice_concentration(os.path.join(os.getcwd(),'Bulkfile.xyz'),os.path.join(os.getcwd(),'Bests-'+Opti.filename+'.xyz'))
                    else:
                        pp.get_lattice_concentration(os.path.join(os.getcwd(),'Bulkfile.xyz'),os.path.join(os.getcwd(),'indiv00.xyz'))
                    os.chdir(cwd)
            Opti.write()
            return
        self.logger.info("Mutating and evolving structures for generation %i" % Opti.generation)
        self.mutate_evolve_write_new_structures(Opti)
        self.logger.info("Changing status back.")
        self.flip_status_back()
        Opti.write()
        return
    
    def evaluate_structures_for_fitness(self, MyOpti):
        """Evaluate completed VASP runs' structures for fitness.
            Args:
                MyOpti <GA Optimizer>
            Returns:
                MyOpti <GA Optimizer>: optimizer with any 
                        settings that had been set
        """
        #Subfolders are complete. Now need to
        #evaluate them for fitness and restart the
        #Optimizer loop if necessary.
        population, offspring = self.get_complete_population(MyOpti)
        self.logger.info(os.environ)
        #evaluate fitness of new individuals in offspring
        for ind in offspring:
            outs = fitness_switch([MyOpti,ind])
            MyOpti.output.write(outs[1])
            ind=outs[0]
        for ind in offspring:
            self.logger.info("Offspring fitness: %3.3f" % ind.fitness)
            #make sure it worked.
        #Now append offspring to population, which includes
        #parents and fitnesses already.
        population.extend(offspring)
        self.logger.info("Opti files: %s" % MyOpti.files)
        pwd = os.getcwd()
        ingpath = self.keywords['name']
        os.chdir(ingpath)
        population = MyOpti.generation_eval(population)
        os.chdir(pwd)
        self.logger.info("Generation incremented to %i" % MyOpti.generation)
        if MyOpti.convergence:
            self.logger.info("Converged!")
            #convokay = open("CONVERGED","wb")
            #convokay.write("Opti.convergence is True at %s\n" % time.asctime())
            #convokay.close()
            #end_signal = MyOpti.algorithm_stats(pop)
            return MyOpti
        else:
            self.logger.info("Clearing folders for next setup.")
            self.clear_folders()
        return MyOpti
    
    def get_complete_population(self, MyOpti):
        self.logger.info("Extracting VASP/LAMMPS outputs.")
        ingpath = self.keywords['name']
        population = list()
        offspring = list()
        pathtooutput = os.path.join(ingpath,MyOpti.filename+'-rank0')
        Totaloutputs = self.extract_output(ingpath)
        tolen = len(Totaloutputs)
        self.logger.info('Length of totaloutputs = {0}'.format(tolen))
        count = 0
        for idx in range(len(MyOpti.population)):
            ind = MyOpti.population[idx]
            if ind.energy != 0:
                population.append(ind)
                count = idx
            else:
                self.logger.info('Count = {0}'.format(count))
                try:
                    (myatoms, myenergy, mypressure) = Totaloutputs[idx-count]
                    passflag = True
                except:
                    passflag = False
                if passflag:
                    if MyOpti.structure=='Defect':
                        from structopt.tools import find_defects
                        outt=find_defects(myatoms.copy(), MyOpti.solidbulk, MyOpti.sf,
                            atomlistcheck=MyOpti.atomlist,trackvacs=MyOpti.trackvacs,
                            trackswaps=MyOpti.trackswaps,debug=False)
                        ind[0]=outt[0]
                        ind.buki=outt[1]
                        ind.vacancies = outt[2]
                        ind.swaps = outt[3]
                        MyOpti.output.write(outt[4])
                    elif MyOpti.structure=='Cluster':
                        myatoms.translate([-MyOpti.large_box_size/2.0,-MyOpti.large_box_size/2.0,-MyOpti.large_box_size/2.0])
                        ind[0] = myatoms.copy()
                    else:
                        ind[0] = myatoms.copy()
                    ind.energy = myenergy
                    ind.pressure = mypressure
                    offspring.append(ind)
        return population, offspring
    
    def extract_output(self, childname=""):
        """Extract output from the VASP/LAMMPS runs.
        """
        outputlist=list()
        keywords = copy.deepcopy(self.keywords)
        keywords['program_keys']['mast_exec'] = self.structopt_parameters['exec_mast']
        for subfolder in self.get_subfolder_list():
            keywords['name'] = os.path.join(childname,subfolder)
            if 'VASP' in self.structopt_parameters['calc_method']:
                mychecker = VaspChecker(name=subfolder, program_keys=self.keywords['program_keys'], structure=self.keywords['structure'])
                mystrfile = os.path.join(subfolder,"CONTCAR")
            elif 'LAMMPS' in self.structopt_parameters['calc_method']:
                mychecker = LammpsChecker(name=subfolder, program_keys=self.keywords['program_keys'], structure=self.keywords['structure'])
                mystrfile = os.path.join(subfolder,"TRAJECTORY")
            mystructure = mychecker.get_structure_from_file(mystrfile)
            myatoms = AseAtomsAdaptor.get_atoms(mystructure)
            myenergy = mychecker.get_energy_from_energy_file()
            mypressure = mychecker.get_final_pressure()
            outputlist.append((myatoms, myenergy, mypressure))
        return outputlist

    def clear_folders(self):
        """Clear the Genetic Algorithm Evaluator ingredient.
            Assumes that the VASP ingredient has already
            been evaluated and the result passed
            to the child ingredient.
            Args:
                ingred_name <str>: Ingredient name to clear.
                    Match this carefully with a name from
                    the recipe.
        """
        fullpath = self.keywords['name']
        self.logger.info("Removing directories and files from ingredient specified at %s" % fullpath)
        timestamp = time.strftime("%Y%m%d_%H%M%S")
        os.mkdir(timestamp)
        for subfolder in self.get_subfolder_list():
            pth = os.path.join(fullpath, timestamp, os.path.basename(subfolder))
            os.renames(subfolder, pth)
        return
    
    def flip_status_back(self):
        """Flip status back to waiting."""
        ingpath = self.keywords['name']
        from MAST.ingredients.chopingredient import ChopIngredient
        cleared_ing = ChopIngredient(name=ingpath, program='None',program_keys = self.keywords['program_keys'], structure=self.keywords['structure'])
        cleared_ing.change_my_status("W")
        return

    def mutate_evolve_write_new_structures(self, MyOpti):
        """Mutate, evolve, and write new structures for VASP.
            Args:
                MyOpti <GA Optimizer>
            Returns:
                MyOpti <GA Optimizer>: optimizer with any 
                        settings that had been set
        """
        #Mutating and evolving
        ingpath = self.keywords['name']
        pathtooutput = os.path.join(ingpath,MyOpti.filename+'-rank0')
        offspring = MyOpti.generation_set(MyOpti.population)
        self.logger.info("generation set; pop len %i" % len(MyOpti.population))
        # Identify the individuals with an invalid fitness
        invalid_ind = [ind for ind in offspring if ind.fitness==0]
        while len(invalid_ind) == 0:
            offspring = MyOpti.generation_set(offspring)
            invalid_ind = [ind for ind in offspring if ind.fitness==0]
        self.run_structures(invalid_ind,MyOpti)
        return MyOpti
    
    def run_structures(self, invalid_ind, MyOpti):
        ingpath = self.keywords['name']
        pathtooutput = os.path.join(ingpath,MyOpti.filename+'-rank0')
        #Delete all old files:
        dirlist = os.listdir(pathtooutput)
        for diritem in dirlist:
            if 'VASP' in self.structopt_parameters['calc_method']:
                if 'POSCAR_' in diritem:
                    os.remove(os.path.join(pathtooutput, diritem))
            elif 'LAMMPS' in self.structopt_parameters['calc_method']:
                if 'DATA' in diritem:
                    os.remove(os.path.join(pathtooutput, diritem))
                if 'atom_symbols' in diritem:
                    os.remove(os.path.join(pathtooutput, diritem))
        #Write structures to POSCAR files or DATA files for evaluation in MAST
        for i in range(len(invalid_ind)):
            if 'VASP' in self.structopt_parameters['calc_method']:
                indname = "%s/POSCAR_%02d" % (MyOpti.filename+'-rank0', i)
                import ase
                if MyOpti.structure == 'Defect':
                    indatoms = invalid_ind[i][0]
                    indatoms.extend(invalid_ind[i].bulki)
                    ase.io.write(indname, indatoms, "vasp", direct=True, sort=True, vasp5=True)
                else:
                    ase.io.write(indname,invalid_ind[i][0],"vasp", direct=True, sort=True, vasp5=True)
            elif 'LAMMPS' in self.structopt_parameters['calc_method']:
                #indname = "%s/DATA_%02d" % (MyOpti.filename+'-rank0', i)
                indname = os.path.join(pathtooutput,'DATA_%02d' % i)
                if MyOpti.structure == 'Defect':
                    indatoms = invalid_ind[i][0]
                    indatoms.extend(invalid_ind[i].bulki)
                    inp_out.write_lammps_data(indname, indatoms)
                elif MyOpti.structure == 'Cluster':
                    indatoms = invalid_ind[i][0]
                    indatoms.set_cell([MyOpti.large_box_size,MyOpti.large_box_size,MyOpti.large_box_size])
                    indatoms.translate([MyOpti.large_box_size/2.0,MyOpti.large_box_size/2.0,MyOpti.large_box_size/2.0])
                    inp_out.write_lammps_data(indname, indatoms)
                else:
                    inp_out.write_lammps_data(indname, invalid_ind[i][0])
                shutil.copy(os.path.join(pathtooutput,'atom_symbols'), 
                    os.path.join(pathtooutput,'atom_symbols_{0:02d}'.format(i)))
#                 shutil.copy("{0}-rank0/atom_symbols".format(MyOpti.filename), 
#                     "{0}-rank0/atom_symbols_{1:02d}".format(MyOpti.filename,i))
            MyOpti.output.write(indname+'\n')
            #Append individuals in invalid_ind to population for writing out
            if MyOpti.generation > 0:
                MyOpti.population.append(invalid_ind[i])
        #Submit list of files to Tam's script to make subfolders
        self.make_subfolders_from_structures(pathtooutput)
        self.set_up_run_folders(pathtooutput)  #Start running.
        return
    
    def make_subfolders_from_structures(self, finddir="", childdir=""):
        """This method makes POSCAR/DATA-containing subfolders from POSCAR/DATA
            files found in a directory
            in childdir if a child directory is given, or in 
            mydir otherwise.
            Args:
                #mydir <str>: Ingredient directory (from self)
                finddir <str>: Directory containing list of structures
                childdir <str>: Child directory (optional)
            Files should be named:
                POSCAR_00 or DATA_00
                POSCAR_01 or DATA_01
                POSCAR_02 or DATA_02
                etc.
        """
        mydir = self.keywords['name']
        logger = logging.getLogger(mydir)
        logger = loggerutils.add_handler_for_recipe(mydir, logger)
        if childdir == "":
            childdir = mydir
        strfiles = os.listdir(finddir)
        for strfile in strfiles:
            if 'VASP' in self.structopt_parameters['calc_method']:
                if (not strfile[0:6] == "POSCAR"):
                    continue
            elif 'LAMMPS' in self.structopt_parameters['calc_method']:
                if (not strfile[0:4] == "DATA"):
                    continue
            if "ase" in strfile:
                continue
            str_path = os.path.join(finddir, strfile)
            subname = strfile.split("_")[1]
            subpath = os.path.join(childdir, subname)
            if not os.path.isdir(subpath):
                os.mkdir(subpath)
            if 'VASP' in self.structopt_parameters['calc_method']:
                shutil.copy(str_path, "%s/POSCAR" % subpath)
            elif 'LAMMPS' in self.structopt_parameters['calc_method']:
                shutil.copy(str_path, "%s/DATA" % subpath)
                shutil.copy(os.path.join(finddir,'atom_symbols_{0}'.format(subname)),
                    "%s/atom_symbols" % subpath)
        return "Wrote all POSCAR/DATA files to subdirectories in %s" % childdir

    def set_up_run_folders(self, childname=""):
        """Set up the Optimizer ingredient subfolders
        """
        self.logger.info("setting up run folders")
        from MAST.ingredients.chopingredient import ChopIngredient
        keywords = self.keywords
        keywords['program_keys']['mast_exec'] = self.structopt_parameters['exec_mast']
        for subfolder in self.get_subfolder_list():
            keywords['name'] = os.path.join(childname,subfolder)
            if 'VASP' in self.structopt_parameters['calc_method']:
                keywords['program'] = 'vasp'
                mychecker = VaspChecker(name=subfolder, program_keys=self.keywords['program_keys'], structure=self.keywords['structure'])
            elif 'LAMMPS' in self.structopt_parameters['calc_method']:
                keywords['program'] = 'lammps'
                mychecker = LammpsChecker(name=subfolder, program_keys=self.keywords['program_keys'], structure=self.keywords['structure'])
            mychecker.set_up_program_input()
            mychoping = ChopIngredient(name=subfolder, program=keywords['program'], program_keys = self.keywords['program_keys'],structure=self.keywords['structure'])
            mychoping.write_submit_script()
            mychoping.run_singlerun()

    def get_subfolder_list(self):
        """Get subfolder list (numeric digits)
        """
        dircontents = os.listdir(self.keywords['name'])
        subfolders = list()
        for diritem in dircontents:
            fulldir = os.path.join(self.keywords['name'],diritem)
            if os.path.isdir(fulldir) and diritem.isdigit():
                subfolders.append(fulldir)
        subfolders.sort()
        return subfolders

    def subfolders_complete(self, childname=""):
        """Check if the Optimizer ingredient subfolders are complete
        """
        from MAST.ingredients.chopingredient import ChopIngredient
        subfolders = self.get_subfolder_list()
        if 'LAMMPS' in self.structopt_parameters['calc_method']:
            count = 2
        else:
            count = 1
        for check in range(count):
            allcomplete=0
            for subfolder in self.get_subfolder_list():
                if 'VASP' in self.structopt_parameters['calc_method']:
                    keywords = self.keywords
                    keywords['program'] = 'vasp'
                    mychecker = VaspChecker(name=subfolder, program_keys=self.keywords['program_keys'], structure=self.keywords['structure'])
                elif 'LAMMPS' in self.structopt_parameters['calc_method']:
                    keywords = self.keywords
                    keywords['program'] = 'lammps'
                    mychecker = LammpsChecker(name=subfolder, program_keys=self.keywords['program_keys'], structure=self.keywords['structure'])
                if mychecker.is_complete():
                    allcomplete = allcomplete + 1
                else:
                    mychoping = ChopIngredient(name=subfolder, program=keywords['program'], program_keys = self.keywords['program_keys'],structure=self.keywords['structure'])
                    mychoping.run_singlerun()
            if allcomplete == len(subfolders) and (allcomplete > 0):
                return True
            if 'LAMMPS' in self.structopt_parameters['calc_method']:
                time.sleep(60)
        return False
    
    def subfolders_complete_looped(self, childname=""):
        """Check if the Optimizer ingredient subfolders are complete
        """
        from MAST.ingredients.chopingredient import ChopIngredient
        subfolders = self.get_subfolder_list()
        allcomplete=0
        for subfolder in self.get_subfolder_list():
            if 'VASP' in self.structopt_parameters['calc_method']:
                keywords = self.keywords
                keywords['program'] = 'vasp'
                mychecker = VaspChecker(name=subfolder, program_keys=self.keywords['program_keys'], structure=self.keywords['structure'])
            elif 'LAMMPS' in self.structopt_parameters['calc_method']:
                keywords = self.keywords
                keywords['program'] = 'lammps'
                mychecker = LammpsChecker(name=subfolder, program_keys=self.keywords['program_keys'], structure=self.keywords['structure'])
            if mychecker.is_complete():
                allcomplete = allcomplete + 1
            else:
                mychoping = ChopIngredient(name=subfolder, program=keywords['program'], program_keys = self.keywords['program_keys'],structure=self.keywords['structure'])
                mychoping.run_singlerun()
        if allcomplete == len(subfolders) and (allcomplete > 0):
            return True
        return False
    
    def convert_asecalc2checker(self):
        return
    
    def get_structures(self, namelist):
        """Get structures from list
            Args:
                #mydir <str>: Ingredient directory (from self)
                namelist <list: List of file names of structures
            Returns:
                return_structures <list of Structure objects>
        """
        import numpy as np
        import pymatgen
        raise NotImplementedError()
        mydir = self.keywords['name']
        logger = logging.getLogger(mydir)
        logger = loggerutils.add_handler_for_recipe(mydir, logger)
        return_structures = list()
        
        for strline in namelist:
            tryline = strline.strip()
            if os.path.isfile(tryline):
                str_path = tryline
            else:
                trypath = os.path.join(mydir, tryline)
                if os.path.isfile(trypath):
                    str_path = trypath
                else:
                    logger.error("No file found at %s" % tryline)
                    continue
            if 'POSCAR' in str_path:
                one_structure = pymatgen.io.smartio.read_structure(str_path)
            elif 'xyz' in str_path:
                myxyz = pymatgen.io.xyzio.XYZ.from_file(str_path)
                allarr = np.array(myxyz.molecule.cart_coords)
                #Get box size:
                maxa=max(allarr[:,0])
                maxb=max(allarr[:,1])
                maxc=max(allarr[:,2])
                boxtol=0.01
                one_structure = myxyz.molecule.get_boxed_structure(maxa+boxtol,maxb+boxtol,maxc+boxtol)
            return_structures.append(one_structure)
        return return_structures

    def forward_final_structure_file(self, childpath, newname="Output-rank0"):
        """Forward the final structure.
            For StructOpt, this is the last population.
            Args:
                childpath <str>: Path of child ingredient
                newname <str>: new name (default 'OUTPUT')
        """
        if 'filename' in self.structopt_parameters:
            newname = self.structopt_parameters['filename']+'-rank0'
        path = os.path.join(childpath,newname)
        os.mkdir(path)
        files = os.listdir(os.path.join(os.getcwd(),newname))
        indivfiles = [one for one in files if ('indiv' in one) and ('iso' not in one)]
        self.logger.info('Found {0} individuals. Passing them to child folder at {1}'.format(len(indivfiles),path))
        for one in indivfiles:
            self.copy_a_file(path, newname+'/'+one, one)
        return
    
    def submit_from_submission_list(self):
        """Submit all entries from the submission list at
            $MAST_CONTROL/submitlist
            Adds a job number to the top of the "jobids" file in each
            ingredient directory.
        """
        control=dirutil.get_mast_control_path()
        submitlist=os.path.join(control, "submitlist")
        if not os.path.isfile(submitlist):
            print "No submission list at %s" % submitlist
            return
        submitfile=MASTFile(submitlist)
        subentries=list(submitfile.data)
        subentries.sort()
        submitted=dict()
        subcommand = "qsub submit.sh" #queue_commands.queue_submission_command()
        for subentry in subentries: # each entry is a directory name
            if len(subentry) == 0:
                continue
            subentry = subentry.strip()
            if len(subentry) == 0:
                continue
            if not os.path.isdir(subentry):
                continue
            elif subentry in submitted.keys():
                continue
            lastjobid = queue_commands.get_last_jobid(subentry)
            jstatus="not yet determined"
            if not (lastjobid == None):
                jstatus = queue_commands.get_job_status_from_queue_snapshot(subentry, lastjobid)
                if jstatus.lower() in ['r','q','h','e']: #running, queued, held, error
                    submitted[subentry]="Already on queue with status %s" % jstatus
                    continue
            os.chdir(subentry)
            subme=subprocess.Popen(subcommand, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            #subme.wait()
            status=subme.communicate()[0]
            queue_commands.write_to_jobids_file(subentry, status)
            submitted[subentry]=status
        queue_commands.print_submitted_dict(submitted)

# if __name__ == "__main__":
#     import sys
#     inputfile = sys.argv[1]
#     Opti = Optimizer(inputfile)
#     Opti.algorithm_initialize()
#     Opti.write()
#     self.mutate_evolve_write_new_structures(Opti)



#!/usr/bin/env python
##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
# Purpose: Modify and get information out of submission scripts.
# 11/9/12 created

import os
import sys
import math #TA to calc num_nodes
import importlib

from MAST.utility.mastfile import MASTFile
from MAST.utility import MASTError

mast_control = os.getenv("MAST_CONTROL")
platform_file = MASTFile("%s/set_platform" % mast_control)
mast_platform = platform_file.data[0].strip()

#mysc = "submit.platforms.script_commands_%s" % mast_platform
#my_script_commands = importlib.import_module(mysc)



"""
Script commands
"""

def write_submit_script(keywords):
    """This script example is built on the following ingredient keywords,
        and may require significant customization.
        mast_processors
        mast_ppn
        mast_queue
        mast_nodes
        mast_exec
        mast_walltime
        mast_memory
    """ 
    import os
    #set defaults
    name = os.path.basename(keywords['name'])
    try:
        mast_processors = str(keywords['program_keys']['mast_processors'])
    except KeyError:
        mast_processors = "8"
    try:
        mast_ppn = str(keywords['program_keys']['mast_ppn'])
    except KeyError:
        mast_ppn = "8"
    try:
        mast_queue = str(keywords['program_keys']['mast_queue'])
    except KeyError:
        mast_queue = 'default'
    try:
        mast_nodes = str(keywords['program_keys']['mast_nodes'])
    except KeyError:
        mast_nodes = "1"
    try:
        mast_exec = str(keywords['program_keys']['mast_exec'])
    except KeyError:
        mast_exec = "mpiexec vasp"
    try:
        mast_walltime = str(keywords['program_keys']['mast_walltime']) + ":00:00"
    except KeyError:
        mast_walltime = "24:00:00"
    try:
        mast_memory = str(keywords['program_keys']['mast_memory'])
    except KeyError:
        mast_memory = "1000"
    #create submission script
    newkey = dict()
    newkey['mast_memory'] = mast_memory
    newkey['mast_exec'] = mast_exec
    newkey['mast_walltime'] = mast_walltime
    newkey['mast_queue'] = mast_queue
    newkey['mast_ppn'] = mast_ppn
    newkey['mast_nodes'] = mast_nodes
    newkey['mast_processors'] = mast_processors
    newkey['mast_name'] = name
    
    my_template = MASTFile("%s/platforms/%s/submit_template.sh" % (mast_control, mast_platform))
    newdata = list()
    for myline in my_template.data:
        for mykey in newkey.keys():
            querykey = "?" + mykey + "?"
            if querykey in myline:
                myline = myline.replace(querykey, newkey[mykey])
        newdata.append(myline)
    filled_template = MASTFile()
    filled_template.data = list(newdata)
    filled_template.to_file("%s/submit.sh" % keywords['name']) #use full path here
    return



from MAST.structopt.tools import get_best
from MAST.structopt.switches import selection_switch

def lambdacommamu(pop, Optimizer):
    """Selection function to employ a lambda,mu GA scheme
    """
    fits = [ind.fitness for ind in pop]
    minindex = [i for i in range(len(pop)) if pop[i].fitness==min(fits)]
    try:
        Optimizer.mark
    except:
        Optimizer.mark = len(pop)/2
    parents = pop[0:Optimizer.mark]
    offspring = pop[Optimizer.mark::]
    if minindex < Optimizer.mark:
        offspring.append(pop[minindex])
    if len(offspring) < Optimizer.nindiv:
        diff = Optimizer.nindiv-len(offspring)
        if Optimizer.natural_selection_scheme=='elitism':
            addins = get_best(parents,diff)
            STR = 'Adding in '+repr(diff)+' lowest fitness parents\n'
        else:
            addins = selection_switch(parents, diff, Optimizer.natural_selection_scheme, Optimizer)
            STR = 'Adding in '+repr(diff)+' parents based on natural selection\n'
        for one in addins:
            offspring.append(one)
    elif len(offspring) > Optimizer.nindiv:
        diff = len(offspring)-Optimizer.nindiv
        if Optimizer.natural_selection_scheme=='elitism':
            offspring = get_best(offspring,Optimizer.nindiv)
            STR = 'Removing lowest '+repr(diff)+' fitness offspring\n'
        else:
            offspring = selection_switch(offspring, Optimizer.nindiv, Optimizer.natural_selection_scheme, Optimizer)
            STR = 'Removing '+repr(diff)+' offspring by natural selection\n'
    else:
        STR = 'Number of offspring = {0}\n'.format(len(offspring))
    return offspring, STR


from MAST.structopt.inp_out import read_xyz
from MAST.structopt.generate.Individual import Individual
from MAST.structopt.tools import find_top_layer

def get_crystal_restart_indiv(Optimizer, indiv):
    """
    Function to generate an structopt Individual class object containing 
        a surface structure from a previously existing structure
    Inputs:
        Optimizer = structopt Optimizer class
        indiv = ASE Atoms object containing the previously existing structure
    Outputs:
        individ = structopt Individual class object containing surface structure data
    *** WARNING: This function is currently degenerate!  ***
    """
    crys = read_xyz(Optimizer.crysfile)
    #Recover cell from Structure Summary file
    # f = open(Optimizer.files[-1],'r')
#     sline = f.readline()
#     lines = f.readlines()
#     popbygen = []
#     n=0
#     for line in lines:
#         if 'Generation' in line:
#             if len(popbygen) != 0:
#                 popbygen.append(genlist)
#             genlist = []
#         else:
#             genlist.append(line)
#     f.close()
    cells = Optimizer.cryscell
    crys.set_cell(cells)
    individ=Individual(crys)
    return individ


import os
from MAST.structopt.tools.lammps import prism

def write_lammps_data(data_name, atoms):
    """Function to write ase atoms object to LAMMPS file
    Input:
        data_name = String or fileobject for data to be written to
        atoms = ase atoms object to be written
    Output:
        None. Data file will be written
    """
    if isinstance(data_name, str):
        f = open(data_name, 'w')
    else:
        f = data_name
    f.write('\n\n')
    symbols = atoms.get_chemical_symbols()
    n_atoms = len(symbols)
    f.write('{0} \t atoms \n'.format(n_atoms))
    #Order species alphabetically
    ordered_symbols = sorted(list(set(symbols)))
    #Write these symbols to a file to ensure they are kept
    dir = os.path.dirname(data_name)
    afilename = os.path.join(dir, 'atom_symbols')
    a = open(afilename,'w')
    for sym in ordered_symbols:
        a.write('{0}\n'.format(sym))
    a.close()
    n_types = len(ordered_symbols)
    f.write('{0}  atom types\n'.format(n_types))
    p = prism(atoms.get_cell())
    xhi, yhi, zhi, xy, xz, yz = p.get_lammps_prism_str()

    f.write('0.0 %s  xlo xhi\n' % xhi)
    f.write('0.0 %s  ylo yhi\n' % yhi)
    f.write('0.0 %s  zlo zhi\n' % zhi)

    if p.is_skewed():
        f.write('%s %s %s  xy xz yz\n' % (xy, xz, yz))
    f.write('\n\n')

    f.write('Atoms \n\n')
    for i, r in enumerate(map(p.pos_to_lammps_str,
                              atoms.get_positions())):
        s = ordered_symbols.index(symbols[i]) + 1
        f.write('%6d %3d %s %s %s\n' % ((i+1, s)+tuple(r)))
    f.close()




from MAST.ingredients.errorhandler.baseerror import BaseError
from MAST.ingredients.errorhandler.vasperror import VaspError
from MAST.ingredients.errorhandler.vaspneberror import VaspNEBError
from MAST.ingredients.errorhandler.phonerror import PhonError
from MAST.ingredients.errorhandler.genericerror import GenericError



#!/usr/bin/env python
##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
# Purpose: Submit a job to the queue. This function is highly platform-specific and may need to be modified.
# Authors: Tam Mayeshiba
# 10/25/12 TTM created
# 12/6/12 TTM added UKY DLX commands
import os
import sys
from MAST.utility import dirutil
import subprocess
import time
from MAST.utility import MASTFile

def direct_shell_command():
    return "vasp"

def queue_submission_command():
    return "sbatch"

def queue_status_from_text(jobid, queuetext):
    """
        Returns the queue status from a line with the job ID information
        INPUTS:
            jobid <int> = job ID
            qstr <str> = queue snapshot line containing job ID
        OUTPUTS:
            <str> = queue status 
            'E': Error
            'X': Not found
            'R': Running
            'Q': Queued
    """
    queuetext = queuetext.strip()
    dlx_out = queuetext.split()[4] #indexing starts at 0
    if (dlx_out == 'PD'):
        final_out = 'Q'
    elif dlx_out in ['CG','CD','R']:
        final_out = 'R'
    return final_out

def extract_submitted_jobid(string):
    """
        Extract the job ID from a string returned when the job is submitted
        INPUTS:
            string <str> = string with job ID somewhere in it
        OUTPUTS:
            <int> = job ID as integer
    """
    findstr = string.strip()
    return int(findstr.split()[3])

def queue_snap_command():
    """
        Return the command for producing a queue snapshot.
        INPUTS:
            None
        OUTPUTS:
            Command for producing a queue snapshot
    """
    uname=os.getenv("USER")
    return "squeue -u %s" % uname

def get_approx_job_error_file(jobid):
    """
        Return the approximate job error file name
            Args:
                jobid <int>: Job ID
    """
    if jobid == None:
        return "slurm"
    else:
        return "slurm-%s.out" % str(jobid)



"""Tests for Recipetemplateparser"""

from MAST.parsers.recipetemplateparser import RecipeTemplateParser

import unittest
from unittest import SkipTest
import os
import time
import MAST
import pymatgen
from MAST.utility import dirutil
from MAST.utility import InputOptions
from MAST.utility import MASTFile
import shutil

testname="recipeparser_test"
testdir = dirutil.get_test_dir(testname)

class TestRecipetemplateparser(unittest.TestCase):

    def setUp(self):
        os.chdir(testdir)
        if not os.path.isdir(os.path.join(testdir,'workdir')):
            os.mkdir(os.path.join(testdir,'workdir'))

    def tearDown(self):
        if os.path.isdir(os.path.join(testdir,'workdir')):
            shutil.rmtree(os.path.join(testdir,'workdir'))

    def test___init__(self):
        rt=os.path.join(testdir,'new_template.txt')
        wd=os.path.join(testdir,'workdir')
        iopt=InputOptions()
        pr=os.path.join(testdir,'workdir','personal_recipe.txt')
        myrtp=RecipeTemplateParser(inputOptions=iopt,working_directory=wd,templateFile=rt,personalRecipe=pr)
        self.assertTrue(os.path.isfile, myrtp.metafile)
        #self.testclass.__init__(**kwargs)

    def test_parse(self):
        #rt=os.path.join(testdir,'new_template.txt')
        rt=os.path.join(testdir,'neb_int.txt')
        wd=os.path.join(testdir,'workdir')
        iopt=InputOptions()
        iopt.options['neb']=dict()
        iopt.options['neb']['nebs']=dict()
        iopt.options['neb']['nebs']['group1-group2']=dict()
        iopt.options['neb']['nebs']['group1-group2']['phonon']=dict()
        iopt.options['neb']['nebs']['group1-group2']['phonon']['solvent']=dict()
        iopt.options['neb']['nebs']['group1-group2']['phonon']['solute']=dict()
        iopt.options['neb']['nebs']['group1-group3']=dict()
        iopt.options['neb']['nebs']['group1-group3']['phonon']=dict()
        iopt.options['defects']=dict()
        iopt.options['defects']['defects']=dict()
        iopt.options['defects']['defects']['group1']=dict()
        iopt.options['defects']['defects']['group1']['charge']=[-1,1,2,3]
        iopt.options['defects']['defects']['group1']['phonon']=dict()
        iopt.options['defects']['defects']['group2']=dict()
        iopt.options['defects']['defects']['group2']['charge']=[1,2,3]
        iopt.options['defects']['defects']['group2']['phonon']=dict()
        iopt.options['defects']['defects']['group3']=dict()
        iopt.options['defects']['defects']['group3']['charge']=[-1]
        iopt.options['defects']['defects']['group3']['phonon']=dict()
        iopt.options['defects']['defects']['group3']['phonon']['host1']=dict()
        pr=os.path.join(testdir,'workdir','personal_recipe.txt')
        myrtp=RecipeTemplateParser(inputOptions=iopt,working_directory=wd,templateFile=rt,personalRecipe=pr)
        rname=myrtp.parse()
        self.assertEquals(len(myrtp.chunks),5)
        mypr = MASTFile(pr)
        for line in mypr.data:
            print line.rstrip()
        compare_pr = MASTFile(os.path.join(testdir,'compare','neb_int_personalized'))
        self.assertEquals(mypr.data, compare_pr.data)
        #self.testclass.parse()

    def test_parse_chunk(self):
        raise SkipTest
        #self.testclass.parse_chunk(chunk)

    def test_old_parsing(self):
        raise SkipTest
        #self.testclass.old_parsing()

    def test_process_system_name(self):
        raise SkipTest
        #self.testclass.process_system_name(processing_lines, system_name)

    def test_process_hop_combinations(self):
        raise SkipTest
        #self.testclass.process_hop_combinations(processing_lines, d_neblines)

    def test_process_images(self):
        raise SkipTest
        #self.testclass.process_images(processing_lines, n_images)

    def test_process_defects(self):
        raise SkipTest
        #self.testclass.process_defects(processing_lines, n_defects, d_defects)

    def test_process_phononlines(self):
        raise SkipTest
        #self.testclass.process_phononlines(processing_lines)

    def test_make_metadata_entries(self):
        raise SkipTest
        #self.testclass.make_metadata_entries(processing_lines)

    def test_get_unique_ingredients(self):
        raise SkipTest
        #self.testclass.get_unique_ingredients()




from MAST.ingredients.checker.basechecker import BaseChecker
from MAST.ingredients.checker.vaspchecker import VaspChecker
from MAST.ingredients.checker.vaspnebchecker import VaspNEBChecker
from MAST.ingredients.checker.phonchecker import PhonChecker
from MAST.ingredients.checker.genericchecker import GenericChecker
from MAST.ingredients.checker.lammpschecker import LammpsChecker
from MAST.ingredients.checker.structoptchecker import StructoptChecker



'''General functions for generating atomic structures'''
# from crystal import *
# from defect import *
from gen_pop_box import *
from gen_pop_sphere import *
from generate_dumbbells import *
from get_population import *
from get_restart_population import *
from Individual import *
# from surface import *
import crystal
import defect
import surface



import sys
import math

#print 'Number of arguments:', len(sys.argv), 'arguments.'

def get_O2_entropy(temp,pressure):
    #Modified from Brian Puchala's MATLAB code:
    
    #Constants
    #All parameters from NIST webbook: http://webbook.nist.gov/cgi/cbook.cgi?ID=C7782447&Units=SI&Type=JANAFG&Table=on#JANAFG
    
    #Boltzmann constant [eV/K]
    k_b = 8.617343e-5
    
    #1eV = 96486 [J/mole]
    eV = 96486
    
    #1 bar = 0.98692 [atm]
    bar = 0.98692
    
    #VASP O2 energy (PAW-PBE O) [eV]
    E_O2 = -9.8644451
    
    #Correction of O2 energy [eV], from Wang et al, PHYSICAL REVIEW B 73, 195107 (2006)
    E_O2_corr = 1.36
    
    temp_mod = temp/1000
    
    if temp < 100:
        print 'Low Temperature!'
        return
    if temp >= 100 and temp < 700:
        A = 31.32234
        B = -20.23531
        C = 57.86644
        D = -36.50624
        E = -0.007374
        F = -8.903471
        G = 246.7945
    if temp >= 700 and temp < 2000:
        A = 30.03235
        B = 8.772972
        C = -3.988133
        D = 0.788313
        E = -0.741599
        F = -11.32468
        G = 236.1663
    if temp >= 2000 and temp <= 6000:
        A = 20.91111
        B = 10.72071
        C = -2.020498
        D = 0.146449
        E = 9.245722
        F = 5.337651
        G = 237.6185
    if temp > 6000:
        print 'High Temperature!'
        return
    #Entropy of oxygen gas (J/mol*K, 298~6000K): Shomate Equation
    S_O2 = ( A*math.log(temp_mod) + B*temp_mod + C*math.pow(temp_mod,2)/2 + D*math.pow(temp_mod,3)/3 - (E/(2*math.pow(temp_mod,2))) + G )/eV
    S_O2_std = 205.0694/eV # S_O2 at temp = 298 K
# if have a temp between 296 K and 300 K, can see that S_O2_std is S_O2 at 298 K:
#    if temp >= 296 and temp <= 300:
#        print temp
#        print S_O2
#        print S_O2_std
    
    #(reference to H_O2_298.15)
    H_O2_NIST = ( A*temp_mod + B*math.pow(temp_mod,2)/2 + C*math.pow(temp_mod,3)/3 + D*math.pow(temp_mod,4)/4 - E/temp_mod + F )*1000/eV

    # Gibbs energy of oxygen gas (G_O2 references to G_O2_298.15)
    G_O2 = ( H_O2_NIST - temp*S_O2 + 298.15*S_O2_std ) + k_b*temp*math.log(pressure/bar)
    
    #Enthalpy of oxygen gas using G=H-TS to obtain H from G.
    H_O2 = ( E_O2 + E_O2_corr ) + G_O2 + ( -298.15*S_O2_std ) + temp*S_O2
    
    #A plain way to get H_O2 is using NIST H_O2 equation and VASP E_O2 plus s correction, which will give the same result. H_O2=(E_O2+E_O2_corr)+ H_O2_NIST
    #print S_O2
    
#    print 'Temp = ', temp, ' [K], Pres = ', pressure, ' [bar], Entropy = ', S_O2, '[eV].'
    print 'Temp = ', temp, ' [K], Entropy = ', S_O2, '[eV].'  
    #return S_O2

#get_O2_entropy(1000,1)
steps = 300
min_temp = 200.
max_temp = 1400.
step_size_temp = ( max_temp - min_temp )/steps
min_press = 1e-9
max_press = 1e-5
step_size_press = ( max_press - min_press )/steps
temp_list = []
press_list = []

for i in xrange(0,steps+1):
    temp_list.append(min_temp + step_size_temp*i)
    press_list.append(min_press + step_size_press*i)

for temp in temp_list:
    pressure = 1e-5
#    for pressure in press_list:
    get_O2_entropy(temp,pressure)



##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-04-25
##############################################################
import sys
import fnmatch
import time
import subprocess
from MAST.utility import MASTError
from MAST.utility.metadata import Metadata
"""General file utilities that do not involve opening a MASTFile
"""

def grepme(filename="", grepstr="", lastlines=""):
    """Use the 'grep' function to grep through a file.
        Args:
            filename <str>: Filename (full path preferred)
            grepstr <str>: String to grep.
            lastlines <str or int>: Search only last X lines.
                Leave blank to grep through entire file.
        Returns:
            Grep results as a list of strings
    """
    if lastlines == "":
        grepcmd = 'grep "%s" %s' % (grepstr, filename)
    else:
        lastlines = str(lastlines)
        grepcmd = 'tail -n %s %s | grep "%s"' % (lastlines, filename, grepstr)
    #print "GREP CMD: %s" % grepcmd
    grepproc = subprocess.Popen(grepcmd, shell=True, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
    grepcomm = grepproc.communicate()[0]
    grepresults = grepcomm.split("\n")
    grepresults.remove("") #Remove trailing carriage return
    return grepresults




import random
from MAST.structopt.tools import get_best

def rank(pop, nkeep, Optimizer):
    """Selection function that chooses structures to survive and orders them based on their relative ranking
    """
    pop = get_best(pop,len(pop))
    newpop = []
    prob = []
    cumprob = []
    for i in range(len(pop)):
        rankprob = float((nkeep-i)) / float(sum(range(nkeep+1)))
        prob.append(rankprob)
        cumprob.append(sum(prob))
    prevcounter = None
    for i in range(nkeep):
        rand = random.random()
        counter = 0
        while cumprob[counter] < rand:
            counter += 1
        if prevcounter==counter:
            while True:
                a = random.choice(pop)
                if a.index != prevcounter:
                    break
            newpop.append(random.choice(pop))
            prevcounter = a.index
        else:
            newpop.append(pop[counter])
            prevcounter = counter
    return newpop


##############################################################
# This code is part of the MAterials Simulation Toolkit (MAST)
# 
# Maintainer: Tam Mayeshiba
# Last updated: 2014-05-12 by Zhewen Song
##############################################################
import os
import time
import fnmatch
import logging

import numpy as np
import pymatgen as pmg

from MAST.utility import InputOptions
from MAST.utility import MASTObj
from MAST.utility import MASTError
from MAST.utility import MAST2Structure
from MAST.utility import dirutil
from MAST.utility import Metadata
from MAST.utility import loggerutils
ALLOWED_KEYS = {\
                 'inputfile'    : (str, 'mast.inp', 'Input file name'),\
               }
MAST_KEYWORDS = {
                 'system_name': 'mast',
                 'mast_auto_correct': 'True'
                }

STRUCTURE_KEYWORDS = {'posfile': None,
                      'spacegroup': None,
                      'symmetry_only': False,
                      'coord_type': 'cartesian',
                      'atom_list': None,
                      'coordinates': None,
                      'lattice': None,
                      'primitive': False,
                      'structure': None
                     }

DEFECTS_KEYWORDS = {'coord_type': 'cartesian',
                    'vacancy': list(),
                    'interstial': list(),
                    'antisite': list(),
                    'substitution': list(),
                   }

INGREDIENTS_KEYWORDS = ['singlepoint',
                        'optimization',
                        'neb',
                   ]

RECIPE_KEYWORDS = {'recipe_file': None,
                  }

class InputParser(MASTObj):
    """Class for parsing a *.inp format input file.
        Attributes:
            self.section_end <str>: delimiter for the end of a section
            self.delimiter <str>: character(s) for breaking up each line
            self.section_parsers <dict>: supported sections
            self.logger <logging Logger>: logger, either recipe-level or MAST input level
    """
    def __init__(self, **kwargs):
        MASTObj.__init__(self, ALLOWED_KEYS, **kwargs)
        self.section_end = '$end'
        self.delimiter = ' ' # how we're breaking up each line
        self.section_parsers = {\
                'mast'     : self.parse_mast_section,
                'structure' : self.parse_structure_section,
                'ingredients' : self.parse_ingredients_section,
                'defects'  : self.parse_defects_section,
                'recipe'   : self.parse_recipe_section,
                'neb'      : self.parse_neb_section,
                'chemical_potentials' : self.parse_chemical_potentials_section,
                'summary' : self.parse_summary_section
                               }
        scratchpath = os.getenv("MAST_SCRATCH").strip('/')
        inputlocation = os.path.dirname(self.keywords['inputfile'])
        if (inputlocation == ""):
            self.logger = logging.getLogger('mast')
            self.logger = loggerutils.add_handler_for_control(self.logger)
        elif scratchpath not in inputlocation:
            self.logger = logging.getLogger('mast')
            self.logger = loggerutils.add_handler_for_control(self.logger)
        else:
            self.logger = logging.getLogger(inputlocation)
            self.logger = loggerutils.add_handler_for_recipe(inputlocation, self.logger)



    def parse(self):
        """Parses information from the *.inp style input file.
            Returns:
                options <InputOptions>: a dictionary of sections and values,
                                        where each section is its own
                                        dictionary:
                                        options.options['section'] = dict()
        """
        options   = InputOptions()
        infile    = file(self.keywords['inputfile'])

        eof = False
        while (not eof):
            line = infile.readline() #.lower() #Remove the lowercasing
            # This works because the blank lines in the file are really 
            # newline (\n)!
            # This is why strip() didn't work, cause it removed that \n, 
            # creating an empty string.
            if (not line):
                eof = True
            elif (line.startswith('#')) or (line.startswith('!')):
            # Lines that start with a # or a ! are treated as a comment and 
            # are skipped
                pass
            elif (line.startswith('$')) and (self.section_end not in line):
                section_name = line[1:].strip()

                if section_name not in self.section_parsers:
                    error = 'Section %s not recognized' % section_name
                    MASTError(self.__class__.__name__, error)
                    return

                section_content = list()
                self.logger.info('Found section %s.  Reading in options.' % section_name)
            elif (self.section_end in line) and (section_name):
                self.section_parsers[section_name](section_name, 
                        section_content, options)
                self.logger.info('Finished parsing the %s section.' % section_name)
            else:
                line = line.strip()
                if (line):
                    section_content.append(line)
        infile.close()

        self.perform_element_mapping(options)
        self.set_structure_from_inputs(options)
        self.logger.info(options)
        self.validate_execs(options)
        return options

    def parse_mast_section(self, section_name, section_content, options):
        """Parses the mast section and populate the options."""
        mast_dict = MAST_KEYWORDS.copy()

        for line in section_content:
            line = line.split(self.delimiter)
            if (line[0] not in mast_dict):
                error = 'Section keyword %s not recognized' % line[0]
                MASTError(self.__class__.__name__, error)
                return
            else:
                mast_dict[line[0]] = line[1]

        for key, value in mast_dict.items():
            options.set_item(section_name, key, value)
        
    def parse_structure_section(self, section_name, section_content, options):
        """Parses the structure section and populate the options.
            Does not create the structure.

            Format is along the lines of:
                coord_type fractional

                begin coordinates
                Ga 0.000000 0.000000 0.000000
                Ga 0.500000 0.500000 0.000000
                Ga 0.000000 0.500000 0.500000
                Ga 0.500000 0.000000 0.500000
                As 0.250000 0.250000 0.250000
                As 0.750000 0.750000 0.250000
                As 0.250000 0.750000 0.750000
                As 0.750000 0.250000 0.750000
                end

                begin lattice
                2.0 0.0 0.0
                0.0 2.0 0.0
                0.0 0.0 2.0
                end

            Note that coord_type will default to "cartesian" if not specified.

        """
        # Initialize with default values
        structure_dict = STRUCTURE_KEYWORDS.copy() 

        subsection_dict = dict()
        for myline in section_content:
            line = myline.split(self.delimiter, 1)

            if (line[0] in structure_dict):
                structure_dict[line[0]] = line[1]
            elif ('begin' in line[0]):
                subsection = line[1]
                subsection_list = list()
            elif ('end' not in line):
                lsplit = myline.split(self.delimiter)
                lineval = list()
                for lval in lsplit:
                    lval.strip()
                    if len(lval) > 0:
                        lineval.append(lval)
                subsection_list.append(lineval)
            elif ('end' in line):
                subsection_dict[subsection] = subsection_list

        # GRJ: Since we lowercase the whole input file, and filenames may not
        # conform to this, we examine the current directory for any files that
        # may match and use that.  If there are multiple matches, we'll throw
        # an error.
        if (structure_dict['posfile'] is not None): # Do we have a geometry file?
            # First build a list of likely files
            origindir = os.getcwd()
            metatry = os.path.join(os.getcwd(), 'metadata.txt')
            if os.path.isfile(metatry):
                myrecipemeta = Metadata(metafile=metatry)
                origindir = myrecipemeta.search_data('origin_dir')[1]
            myfiles = dirutil.walkfiles(origindir)
            file_list=list()
            for myfile in myfiles:
                if structure_dict['posfile'] in myfile:
                    file_list.append(myfile)
            if (len(file_list) > 1):
                # If we have multiple files with the same name, but different capitalization, throw an error here
                self.logger.warning('Found multiple files with the name %s' % structure_dict['posfile'])
                self.logger.warning('Found the files:')
                for file in file_list:
                    self.logger.warning(file)
                error='Found ambiguous file names'
                raise MASTError(self.__class__.__name__, error)
            elif len(file_list) == 0:
                raise MASTError(self.__class__.__name__, "No structure file %s found in %s" % (structure_dict['posfile'], origindir))
            else:
                structure_dict['posfile'] = file_list[0]

        # print 'in InputParser.parse_structure_section:', subsection_dict
        # TM
        element_map = dict()
        atom_list = list()
        scaling = dict()
        for key, value in subsection_dict.items():
            if (key == 'coordinates'):
                value = np.array(value)
                # Here we use .title() to re-capitalize the first letter of all 
                # the atomic symbols to comply with what
                # pymatgen needs
                atom_list = [val.title() for val in value[:, 0]]
                structure_dict['atom_list'] = atom_list
                coordinates = np.array(value[:, 1:], dtype='float')
                structure_dict['coordinates'] = coordinates
            if (key == 'lattice'):
                #print "VALUE: ", value
                lattice = np.array(value, dtype='float')
                structure_dict['lattice'] = lattice
            if (key == 'elementmap'):
                for elline in value:
                    elkey = elline[0].strip().upper() #all caps
                    elname = elline[1].strip().title() #Title case
                    element_map[elkey]=elname
                structure_dict['element_map'] = element_map
            if (key == 'scaling'):
                for scline in value:
                    scsize = scline[0].strip()
                    sckmesh = scline[1].strip()
                    scktype = scline[2].strip()
                    sckshift = "0.0 0.0 0.0"
                    try: sckshift = "%s %s %s"%(scline[3],scline[4],scline[5])
                    except IndexError: pass
                    scaling[scsize]=[sckmesh,scktype,sckshift]
                structure_dict['scaling'] = scaling
        if len(element_map) > 0 and len(atom_list) > 0:
            new_atom_list = list()
            for atomval in atom_list:
                if atomval.upper() in element_map.keys():
                    new_atom_list.append(element_map[atomval])
                else:
                    new_atom_list.append(atomval)
            structure_dict['atom_list'] = new_atom_list

        for key, value in structure_dict.items():
            options.set_item(section_name, key, value)

    def parse_defects_section(self, section_name, section_content, options):
        """Parses the defects section and populates the options.
        """
        defect_list = ['antisite', 'vacancy', 'substitution', 'interstitial']
        defect_types = dict()
        multidefect = False
        charge = [0]
        count = 1
        coord_type = 'cartesian'
        threshold = 1.e-4

        for line in section_content:
            line = line.split(self.delimiter)

            if (line[0] == 'coord_type'):
                coord_type = line[1]
            elif (line[0] == 'threshold'):
                threshold = float(line[1])
            elif (line[0] in defect_list) and (not multidefect):
                type_dict = dict()
                label = None
                charge = [0]

                if (len(line) < 5):
                    error = 'Defect specification requires at least 5 arguments.'
                    MASTError(self.__class__.__name__, error)

                # Check for static options
                type_dict['type'] = line[0]
                coord = line[1:4]
                type_dict['coordinates'] = np.array(coord, dtype='float')
                type_dict['symbol'] = line[4]

                if (len(line) > 5):
                    for lin in line[5:]:
                        lin = lin.split('=')
                        if (lin[0] == 'charge'):
                            charge_range = lin[1].split(',')
                            charge = range(int(charge_range[0]),
                                           int(charge_range[1])+1)
                        elif (lin[0] == 'label'):
                            label = lin[1]

                if (not label):
                    label = 'defect_%s' % str(count)

                defect = {'charge': charge,
                          'subdefect_1': type_dict,
                          'coord_type': coord_type,
                          'threshold': threshold,
                          'phonon': dict()}

                #defect_types['defect_%s' % label] = defect
                defect_types[label] = defect

                count += 1
            elif (line[0] == 'begin'):
                defect = dict()
                multidefect = True
                subcount = 1
                defect['charge'] = charge
                defect['coord_type'] = coord_type
                defect['threshold'] = threshold
                defect['phonon'] = dict()

                try:
                    label = line[1]
                except IndexError:
                    label = None
            elif (line[0] == 'end'):
                if (not label):
                    label = 'defect_%s' % str(count)

                defect_types[label] = defect
                count += 1
                multidefect = False
            elif ('charge' in line[0]):
                charge_range = line[0].split('=')[-1].split(',')
                defect['charge'] = range(int(charge_range[0]),
                                         int(charge_range[1])+1)
            elif ('phonon' in line[0]):
                plabel = line[1]
                p_center = ' '.join(line[2:5])
                p_radius = float(line[5])
                p_thresh = 0.1
                if len(line) > 6:
                    p_thresh = float(line[6])
                defect['phonon'][plabel]=dict()
                defect['phonon'][plabel]['phonon_center_site'] = p_center
                defect['phonon'][plabel]['phonon_center_radius'] = p_radius
                defect['phonon'][plabel]['threshold'] = p_thresh
            else:
                type_dict = dict()

                if (len(line) < 5):
                    error = 'Defect specification requires at least 5 arguments.'
                    MASTError(self.__class__.__name__, error)

                # Check for static options
                type_dict['type'] = line[0]
                coord = line[1:4]
                type_dict['coordinates'] = np.array(coord, dtype='float')
                type_dict['symbol'] = line[4]

                defect['subdefect_%i' % subcount] = type_dict
                # print 'Rawr!', defect
                subcount += 1
        options.set_item(section_name, 'num_defects', count-1)
        options.set_item(section_name, 'defects', defect_types)
        options.set_item(section_name, 'coord_type', coord_type)

    def parse_recipe_section(self, section_name, section_content, options):
        """Parses the recipe section and populates the options."""
        recipe_dict = RECIPE_KEYWORDS.copy()

        for line in section_content:
            line = line.split(self.delimiter)
            if (line[0] not in recipe_dict):
                error = 'Section keyword %s not recognized' % line[0]
                MASTError(self.__class__.__name__, error)
                return
            elif (line[0] == 'recipe_file'):
                try:
                    recipe_path = os.getenv('MAST_RECIPE_PATH')
                except KeyError:
                    error = 'MAST_RECIPE_PATH environment variable not set'
                    MASTError(self.__class__.__name__, error) 
                recipe_dict['recipe_file'] = '%s/%s' % (recipe_path, line[1])
            else:
                recipe_dict[line[0]] = line[1]
        for key, value in recipe_dict.items():
            options.set_item(section_name, key, value)

    def parse_ingredients_section(self, section_name, section_content, options):
        """Parses the ingredients section and populates the options.
            Section takes the form of:
                $ingredients
                begin ingredients_global
                mast_kpoints 3x3x3
                mast_xc pbe
                end

                begin singlepoint
                encut 400
                end

                begin optimize
                encut 300
                ibrion 2
                end

                $end

            mast_kpoints are parsed out as a 3 index list of integers. 
            Everything else is parsed out as a string.

            Anything in ingredients_global is then appended onto each 
            individual ingredient.
        """

        global_dict = dict()
        ingredients_dict = dict()

        for line in section_content:
            if (line.startswith('begin')):
                # Each ingredient section starts with "begin".
                # Check for this line, and initialize the individual
                # ingredient dictionary
                ingredient_name = line.split()[1]
                ingredient_dict = dict()
            elif (not (line == 'end')):
                opt = line.split()
                # print opt
                if (opt[0] == 'mast_kpoints'):
                    kpts = map(int, opt[1].split('x'))
                    # Second option after mast_kpoints tells where to 
                    # center the k-point mesh.
                    # If it's there, we append it onto the k-point grid list.
                    if len(opt) > 2:
                        kpts.append(opt[2])
                    ingredient_dict[opt[0]] = kpts
                elif (opt[0] == 'mast_pp_setup'):
                    psp_dict = dict()
                    for key in opt[1:]:
                        key = key.split('=')
                        ref = key[0].title()
                        val = str().join(key[1][0].upper() + key[1][1:])
                        psp_dict[ref] = val
                    ingredient_dict[opt[0]] = psp_dict
                #elif (opt[0] == 'mast_exec'):
                #    ingredient_dict[opt[0]] = ' '.join(opt[1:]) #preserve whole line
                #elif (opt[0] == 'mast_strain'):
                #    ingredient_dict[opt[0]] = ' '.join(opt[1:]) #preserve whole line
                #elif (opt[0] == 'ptemp'):
                #    ingredient_dict[opt[0]] = ' '.join(opt[1:]) #preserve whole line 
                #elif (opt[0] == 'rwigs'):
                #    ingredient_dict[opt[0]] = ' '.join(opt[1:]) #preserve whole line
                #elif (opt[0] == 'mast_setmagmom'):
                #    ingredient_dict[opt[0]] = ' '.join(opt[1:]) #preserve whole line
                elif (opt[0] == 'mast_coordinates'):
                    shortsplit = opt[1].split(",")
                    filesplit=list()
                    origindir = os.getcwd()
                    metatry = os.path.join(os.getcwd(), 'metadata.txt')
                    if os.path.isfile(metatry):
                        myrecipemeta = Metadata(metafile=metatry)
                        origindir = myrecipemeta.search_data('origin_dir')[1]
                    myfiles = dirutil.walkfiles(origindir)
                    for shortname in shortsplit:
                        for fullfile in myfiles:
                            if shortname.strip() in os.path.basename(fullfile):
                                filesplit.append(fullfile)
                    if not (len(filesplit) == len(shortsplit)):
                        raise MASTError(self.__class__.__name__, "Not all files given by %s were found in %s." % (shortsplit, origindir))
                    ingredient_dict[opt[0]] = filesplit
                else:
                    ingredient_dict[opt[0]] = ' '.join(opt[1:]) #preserve whole line
                    #ingredient_dict[opt[0]] = opt[1] #old behavior took only the first part
                if (opt[0] == 'mast_program') and (opt[1] == 'vasp' or opt[1] == 'vasp_neb'):
                    if os.getenv('VASP_PSP_DIR') == None:
                        raise MASTError(self.__class__.__name__, "Input file specifies program vasp, but no POTCAR directory is set in environment variable VASP_PSP_DIR")
            elif ('end' in line):
                # Each ingredient section ends with "end", if present finish 
                # that current section and assign
                # the neccessary element in the ingredients dictionary and 
                # create the global dictionary
                if (ingredient_name == 'ingredients_global'):
                    global_dict = ingredient_dict
                else:
                    ingredients_dict[ingredient_name] = ingredient_dict

        # Each value in ingredients_dict is a dictionary containing the relevant
        # ingredient and option(s).
        for ing_key, ing_value in ingredients_dict.items():
            options.set_item(section_name, ing_key, ing_value)

        options.set_item(section_name, 'global', global_dict)

    def parse_neb_section(self, section_name, section_content, options):
        """Parses the neb section and populates the options.
            
            The number of images is specified and must be the same
            throughout the recipe.

            Each neb gets a label which should correspond to a defect group
            or defect group number.

            Within each subsection, each line specifies which atom is moving,
            giving its element name and its approximate coordinates from
            the starting unrelaxed structure.

            $neb

            images 3

            begin crowdion1-crowdion2
            Cr, 0 0.3 0, 0 0 0
            Ni, 0 0.6 0, 0 0.3 0
            end

            begin int1-int2
            Mg, 0 0 0.5, 0.3 0.1 2
            end

            $end

            In the crowdion example, the starting undefected structure is
            defected three times in each defect group: 1 vacancy and two
            antisites. The neb section allows us to unambiguously match up 
            the atoms and determine which atoms are moving, and where. 
        """
        nebs = dict()
        images = 0
        neblabel=""
        for line in section_content:
            line = line.strip()
            #if 'images' in line:
            #    line = line.split(self.delimiter)
            #    images = int(line[1])
            if 'begin' in line:
                line = line.split(self.delimiter)
                neblabel = line[1].strip()
                nebs[neblabel]=dict()
                nebs[neblabel]['lines']=list()
                nebs[neblabel]['phonon']=dict()
            elif 'end' in line:
                neblabel = ""
            else:
                line = line.split(',') #use commas for element lines
                if 'phonon' in line[0]:
                    sline = line[0].split()
                    plabel = sline[1]
                    p_center = ' '.join(sline[2:5])
                    p_radius = float(sline[5])
                    p_thresh = 0.1
                    if len(sline) > 6:
                        p_thresh = float(sline[6])
                    nebs[neblabel]['phonon'][plabel]=dict()
                    nebs[neblabel]['phonon'][plabel]['phonon_center_site'] = p_center
                    nebs[neblabel]['phonon'][plabel]['phonon_center_radius'] = p_radius
                    nebs[neblabel]['phonon'][plabel]['threshold'] = p_thresh
                elif 'images' in line[0]:
                    nebs[neblabel]['images'] = int(line[0].split()[1])
                else:
                    nebs[neblabel]['lines'].append(line)

        #options.set_item(section_name, 'images', images)
        options.set_item(section_name, 'nebs', nebs)

    def parse_chemical_potentials_section(self, section_name, section_content, options):
        """Parses the chemical_potentials section and populates the options.
            Section uses the standard begin...end subsection structure, but with
            a modification: instead of strict subsection titles (i.e. structure,
            lattice etc.), subsection titles are the conditions under which the
            chemical potentials are for.  Any combination of white spacing is
            allowed, however note that all conditions will be converted to lower
            case first!

            Using a GaAs example, hypothetically we could have a 
            chemical_potential section like the following:
                $chemical_potentials
                begin As rich
                Ga 4.5
                As 3.5
                end

                begin Ga rich
                Ga 3.5
                As 4.5
                end
                $end

            For charged defects, note that the chemical potential of the
            electron will be calculated automatically from the Fermi level and
            appropiate potential shift correction.  However, if one desires to
            manually give an electron chemical potential (for whatever reason!)
            this can be done by specifiying \'electron\' in the appropiate
            condition.
        """
        chempot_dict = dict()

        for line in section_content:
            if (line.startswith('begin')):
                condition_name = ' '.join(line.split()[1:])
                condition_dict = dict()
            elif ('end' not in line):
                opt = line.split()
                condition_dict[opt[0]] = float(opt[1])
            elif ('end' in line):
                chempot_dict[condition_name] = condition_dict

        for key, value in chempot_dict.items():
            options.set_item(section_name, key, value)


    def parse_phonon_section(self, section_name, section_content, options):
        """Parses the phonon section and populates the options.
            
            The phonon_center_site is a coordinate (as a string, like "0 0 0")
                for the center of the phonon calculations. Give a coordinate
                of an atom for that atom.
                Do not set this value in order to calculate phonons for all
                atoms.
            The phonon_center_radius is a radius in ANGSTROMS to search for
                neighbors to the phonon_center_site, and also take into account
                their vibrations.
                Do not set this value or set to 0 to only calculate phonons
                for the atom given in phonon_center_site.
            All phonons must be enclosed in a begin and end tag within the 
            phonon section. Phonons should be labeled
            xxxx_phonon_label and the label should correspond to the 
            auto-generation of the
            recipe (e.g. corresponding exactly to defect group names like vac1
            or to neb image names like vac1-vac2_02)

            $phonon

            begin crowdion1
            phonon_center_site 0 0.3 0
            phonon_center_radius 2
            end
            
            begin crowdion1-crowdion2_02
            phonon_center_site 0 0.15 0
            phonon_center_radius 2
            end

            $end
        """
        raise MASTError(self.__class__.__name__,"Phonon section in input files *.inp is obsolete. Phonons now go in the defects and neb sections.")
        phonon = dict()
        label=""
        for line in section_content:
            line = line.strip()
            if 'begin' in line:
                line = line.split(self.delimiter)
                label = line[1].strip()
                phonon[label]=dict()
            elif 'end' in line:
                pass
            else:
                line = line.split(self.delimiter, 1)
                phonon[label][line[0]] = line[1]

        options.set_item(section_name, 'phonon', phonon)


    def perform_element_mapping(self, input_options):
        """Perform element mapping if there is an element map specified,
            so that defects and NEB lines get the correct element name.
            Args:
                input_options <InputOptions>
            Returns:
                modifies input_options dictionary entries in place.
        """
        #print self.input_options.get_section_keys('structure')
        eldict=dict()
        if 'element_map' in input_options.get_section_keys('structure'):
            eldict = input_options.get_item('structure','element_map')
        if len(eldict) == 0:
            return
        if 'defects' in input_options.get_sections():
            defdict = input_options.get_item('defects','defects')
            for dkey in defdict.keys():
                for sdkey in defdict[dkey].keys():
                    if 'subdefect' in sdkey:
                        symbol = defdict[dkey][sdkey]['symbol'].upper()
                        if symbol in eldict.keys():
                            defdict[dkey][sdkey]['symbol'] = eldict[symbol]
        if 'neb' in input_options.get_sections():
            nebdict = input_options.get_item('neb','nebs')
            for nebkey in nebdict.keys():
                nlinenum = len(nebdict[nebkey]['lines'])
                nlinect=0
                while nlinect < nlinenum:
                    symbol = nebdict[nebkey]['lines'][nlinect][0].upper()
                    if symbol in eldict.keys():
                        nebdict[nebkey]['lines'][nlinect][0] = eldict[symbol]
                    nlinect = nlinect + 1
        return



    def validate_execs(self, input_options):
        """Make sure each ingredient has a mast_exec line.
            Args:
                input_options <InputOptions>
        """
        have_exec = False
        for ingredient, options in input_options.get_item('ingredients').items():
            if 'mast_exec' in options:
                have_exec = True
                break
            else:
                have_exec = False

        if (not have_exec):
            error = 'mast_exec keyword not found in the $ingredients section'
            raise MASTError(self.__class__.__name__, error)
    def set_structure_from_inputs(self, input_options):
        """Make a pymatgen structure and update the
            structure key.
            Args:
                input_options <InputOptions>
        """
        strposfile = input_options.get_item('structure','posfile')
        if strposfile is None:
            iopscoords=input_options.get_item('structure','coordinates')
            iopslatt=input_options.get_item('structure','lattice')
            iopsatoms=input_options.get_item('structure','atom_list')
            iopsctype=input_options.get_item('structure','coord_type')
            structure = MAST2Structure(lattice=iopslatt,
                coordinates=iopscoords, atom_list=iopsatoms,
                coord_type=iopsctype)
        elif ('poscar' in strposfile.lower()):
            from pymatgen.io.vaspio import Poscar
            structure = Poscar.from_file(strposfile).structure
        elif ('cif' in strposfile.lower()):
            from pymatgen.io.cifio import CifParser
            structure = CifParser(strposfile).get_structures()[0]
        else:
            error = 'Cannot build structure from file %s' % strposfile
            raise MASTError(self.__class__.__name__, error)
        input_options.update_item('structure','structure',structure)

    def parse_summary_section(self, section_name, section_content, options):
        """Parses the summary section and populates the options."""
        mast_dict = dict()
        for line in section_content:
            line = line.split(self.delimiter, 1)
            mast_dict.setdefault(line[0], []).append(line[1].strip())

        for key, value in mast_dict.items():
            options.set_item(section_name, key, value)



'''General functions for input/output use in structopt'''
from read_parameter_input import *
from read_xyz import *
from setup_output import *
from setup_par_output import *
from write_parameters import *
from write_pop import write_pop
from write_xyz import *
from write_optimizer import *
from restart_output import *
from read_individual import *
from write_individual import *
from write_lammps_data import *


#Fake vasp for testing Amy's GA:
import numpy as np
import time
import pymatgen
import ase
from ase import io as aseio
timestamp = time.asctime()
randenergy = np.random.random_sample()*-100.0 + -200.0
randtime = np.random.random_sample()*100
randpress = np.random.random_sample()*50

#mystruc = pymatgen.io.vaspio.Poscar.from_file("POSCAR").structure
#mystruc.perturb(0.01)
#fake_contcar = pymatgen.io.vaspio.Poscar(mystruc)
#fake_contcar.write_file("CONTCAR",direct=False)
#
from ase.calculators.lj import LennardJones
from ase.optimize import BFGS
indiv = aseio.read("POSCAR")
calc = LennardJones()
indiv.set_calculator(calc)
indiv.set_pbc=False
dyn=BFGS(indiv)
try:
    dyn.run(fmax=0.01, steps=1000)
    aseenergy = indiv.get_potential_energy()
    asepress = indiv.get_isotropic_pressure(indiv.get_stress())
except:
    aseenergy = 10000
    asepress = 10000
indiv.set_pbc=True


#poscaratoms.set_cell([4.275, 4.275, 4.275])
ase.io.write("CONTCAR",indiv, "vasp", direct=True, sort=True, vasp5=True)

fake_osz = open("OSZICAR","wb")
fake_osz.write("Output randomly generated at %s\n" % timestamp)
#fake_osz.write("E0=% 3.8f   d E =0.00\n" % randenergy)  
fake_osz.write("E0=% 3.8f   d E =0.00\n" % aseenergy)  
fake_osz.close()

fake_outcar = open("OUTCAR","wb")
fake_outcar.write("Output randomly generated at %s\n" % timestamp)
#fake_outcar.write("external pressure =   %3.8f kB  Pullay stress =  0.00 kB\n" % randpress)
fake_outcar.write("external pressure =   %3.8f kB  Pullay stress =  0.00 kB\n" % asepress)
fake_outcar.write("Pretend we have...reached required accuracy\n")
fake_outcar.write("User time (sec): %3.8f" % randtime)
fake_outcar.close()

